@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Function_Type.cxx 6013 2008-01-04 22:18:40Z matt $"
+// "$Id: Fl_Function_Type.cxx 8786 2011-06-07 11:41:36Z manolo $"
 //
 // C function type code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2011 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,6 +30,7 @@
 #include <FL/Fl_File_Chooser.H>
 #include "Fl_Type.h"
 #include <FL/fl_show_input.H>
+#include <FL/Fl_File_Chooser.H>
 #include "../src/flstring.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,7 +42,11 @@ extern const char* i18n_file;
 extern const char* i18n_set;
 extern char i18n_program[];
 
+extern int compile_only;
+
 extern void redraw_browser();
+extern void goto_source_dir();
+extern void leave_source_dir();
 
 ////////////////////////////////////////////////////////////////
 // quick check of any C code for legality, returns an error message
@@ -51,14 +56,14 @@ static char buffer[128]; // for error messages
 // check a quoted string ending in either " or ' or >:
 const char *_q_check(const char * & c, int type) {
   for (;;) switch (*c++) {
-  case '\0':
-    sprintf(buffer,"missing %c",type);
-    return buffer;
-  case '\\':
-    if (*c) c++;
-    break;
-  default:
-    if (*(c-1) == type) return 0;
+    case '\0':
+      sprintf(buffer,"missing %c",type);
+      return buffer;
+    case '\\':
+      if (*c) c++;
+      break;
+    default:
+      if (*(c-1) == type) return 0;
   }
 }
 
@@ -66,50 +71,50 @@ const char *_q_check(const char * & c, int type) {
 const char *_c_check(const char * & c, int type) {
   const char *d;
   for (;;) switch (*c++) {
-  case 0:
-    if (!type) return 0;
-    sprintf(buffer, "missing %c", type);
-    return buffer;
-  case '/':
-    // Skip comments as needed...
-    if (*c == '/') {
-      while (*c != '\n' && *c) c++;
-    } else if (*c == '*') {
-      c++;
-      while ((*c != '*' || c[1] != '/') && *c) c++;
-      if (*c == '*') c+=2;
-      else {
-        return "missing '*/'";
+    case 0:
+      if (!type) return 0;
+      sprintf(buffer, "missing %c", type);
+      return buffer;
+    case '/':
+      // Skip comments as needed...
+      if (*c == '/') {
+        while (*c != '\n' && *c) c++;
+      } else if (*c == '*') {
+        c++;
+        while ((*c != '*' || c[1] != '/') && *c) c++;
+        if (*c == '*') c+=2;
+        else {
+          return "missing '*/'";
+        }
       }
-    }
-    break;
-  case '#':
-    // treat cpp directives as a comment:
-    while (*c != '\n' && *c) c++;
-    break;
-  case '{':
-    if (type==')') goto UNEXPECTED;
-    d = _c_check(c,'}');
-    if (d) return d;
-    break;
-  case '(':
-    d = _c_check(c,')');
-    if (d) return d;
-    break;
-  case '\"':
-    d = _q_check(c,'\"');
-    if (d) return d;
-    break;
-  case '\'':
-    d = _q_check(c,'\'');
-    if (d) return d;
-    break;
-  case '}':
-  case ')':
-  UNEXPECTED:
-    if (type == *(c-1)) return 0;
-    sprintf(buffer, "unexpected %c", *(c-1));
-    return buffer;
+      break;
+    case '#':
+      // treat cpp directives as a comment:
+      while (*c != '\n' && *c) c++;
+      break;
+    case '{':
+      if (type==')') goto UNEXPECTED;
+      d = _c_check(c,'}');
+      if (d) return d;
+      break;
+    case '(':
+      d = _c_check(c,')');
+      if (d) return d;
+      break;
+    case '\"':
+      d = _q_check(c,'\"');
+      if (d) return d;
+      break;
+    case '\'':
+      d = _q_check(c,'\'');
+      if (d) return d;
+      break;
+    case '}':
+    case ')':
+    UNEXPECTED:
+      if (type == *(c-1)) return 0;
+      sprintf(buffer, "unexpected %c", *(c-1));
+      return buffer;
   }
 }
 
@@ -178,10 +183,12 @@ void Fl_Function_Type::open() {
     f_public_member_choice->hide();
   }
   f_c_button->value(cdecl_);
+  const char *c = comment();
+  f_comment_input->buffer()->text(c?c:"");
   function_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == f_panel_cancel) goto BREAK2;
@@ -218,10 +225,19 @@ void Fl_Function_Type::open() {
       mod = 1;
       cdecl_ = f_c_button->value();
     }
+    c = f_comment_input->buffer()->text();
+    if (c && *c) {
+      if (!comment() || strcmp(c, comment())) redraw_browser();
+      comment(c);
+    } else {
+      if (comment()) redraw_browser();
+      comment(0);
+    }
+    if (c) free((void*)c);
     if (mod) set_modflag(1);
     break;
   }
- BREAK2:
+BREAK2:
   function_panel->hide();
 }
 
@@ -233,15 +249,23 @@ void Fl_Function_Type::write_code1() {
   constructor=0;
   havewidgets = 0;
   Fl_Type *child;
-  for (child = next; child && child->level > level; child = child->next)
+  // if the function has no children (hence no body), Fluid will not generate
+  // the function either. This is great if you decide to implement that function
+  // inside another module
+  char havechildren = 0;
+  for (child = next; child && child->level > level; child = child->next) {
+    havechildren = 1;
     if (child->is_widget()) {
       havewidgets = 1;
       break;
     }
-  write_c("\n");
-  if (ismain())
-    write_c("int main(int argc, char **argv) {\n");
-  else {
+  }
+  if (havechildren)
+    write_c("\n");
+  if (ismain()) {
+    if (havechildren)
+      write_c("int main(int argc, char **argv) {\n");
+  } else {
     const char* rtype = return_type;
     const char* star = "";
     // from matt: let the user type "static " at the start of type
@@ -256,109 +280,118 @@ void Fl_Function_Type::write_code1() {
     }
     if (!rtype) {
       if (havewidgets) {
-	rtype = subclassname(child);
-	star = "*";
+        rtype = subclassname(child);
+        star = "*";
       } else rtype = "void";
     }
-
+    
     const char* k = class_name(0);
     if (k) {
+      if (havechildren)
+        write_comment_c();
       write_public(public_);
       if (name()[0] == '~')
-	constructor = 1;
+        constructor = 1;
       else {
-	size_t n = strlen(k);
-	if (!strncmp(name(), k, n) && name()[n] == '(') constructor = 1;
+        size_t n = strlen(k);
+        if (!strncmp(name(), k, n) && name()[n] == '(') constructor = 1;
       }
       write_h("  ");
       if (is_static) write_h("static ");
       if (is_virtual) write_h("virtual ");
       if (!constructor) {
         write_h("%s%s ", rtype, star);
-	write_c("%s%s ", rtype, star);
+        if (havechildren)
+          write_c("%s%s ", rtype, star);
       }
-
+      
       // if this is a subclass, only write_h() the part before the ':'
       char s[1024], *sptr = s;
       char *nptr = (char *)name();
-
+      
       while (*nptr) {
         if (*nptr == ':') {
-	  if (nptr[1] != ':') break;
-	  // Copy extra ":" for "class::member"...
+          if (nptr[1] != ':') break;
+          // Copy extra ":" for "class::member"...
           *sptr++ = *nptr++;
         }	  
         *sptr++ = *nptr++;
       }
       *sptr = '\0';
-
+      
       write_h("%s;\n", s);
       // skip all function default param. init in body:
       int skips=0,skipc=0;
       int nc=0,plevel=0;
       for (sptr=s,nptr=(char*)name(); *nptr; nc++,nptr++) {
-	if (!skips && *nptr=='(') plevel++;
-	else if (!skips && *nptr==')') plevel--;
-	if ( *nptr=='"' &&  !(nc &&  *(nptr-1)=='\\') ) 
-	  skips = skips ? 0 : 1;
-	else if(!skips && *nptr=='\'' &&  !(nc &&  *(nptr-1)=='\\'))
-	  skipc = skipc ? 0 : 1;
-	if(!skips && !skipc && plevel==1 && *nptr =='=' && 
-	   !(nc && *(nptr-1)=='\'') ) // ignore '=' case 
-	  while(*++nptr  && (skips || skipc || (*nptr!=',' && *nptr!=')' || plevel!=1) )) {
-	    if ( *nptr=='"' &&  *(nptr-1)!='\\' ) 
-	      skips = skips ? 0 : 1;
-	    else if(!skips && *nptr=='\'' &&  *(nptr-1)!='\\')
-	      skipc = skipc ? 0 : 1;
-	    if (!skips && !skipc && *nptr=='(') plevel++;
-	    else if (!skips && *nptr==')') plevel--;
-	  }
-
-	if (sptr < (s + sizeof(s) - 1))	*sptr++ = *nptr;
+        if (!skips && *nptr=='(') plevel++;
+        else if (!skips && *nptr==')') plevel--;
+        if ( *nptr=='"' &&  !(nc &&  *(nptr-1)=='\\') ) 
+          skips = skips ? 0 : 1;
+        else if(!skips && *nptr=='\'' &&  !(nc &&  *(nptr-1)=='\\'))
+          skipc = skipc ? 0 : 1;
+        if(!skips && !skipc && plevel==1 && *nptr =='=' && 
+           !(nc && *(nptr-1)=='\'') ) // ignore '=' case 
+          while(*++nptr  && (skips || skipc || ( (*nptr!=',' && *nptr!=')') || plevel!=1) )) {
+            if ( *nptr=='"' &&  *(nptr-1)!='\\' ) 
+              skips = skips ? 0 : 1;
+            else if(!skips && *nptr=='\'' &&  *(nptr-1)!='\\')
+              skipc = skipc ? 0 : 1;
+            if (!skips && !skipc && *nptr=='(') plevel++;
+            else if (!skips && *nptr==')') plevel--;
+          }
+        
+        if (sptr < (s + sizeof(s) - 1))	*sptr++ = *nptr;
       }
       *sptr = '\0';
- 
-      write_c("%s::%s {\n", k, s);
+      
+      if (havechildren)
+        write_c("%s::%s {\n", k, s);
     } else {
+      if (havechildren)
+        write_comment_c();
       if (public_) {
-	if (cdecl_)
-	  write_h("extern \"C\" { %s%s %s; }\n", rtype, star, name());
-	else
-	  write_h("%s%s %s;\n", rtype, star, name());
+        if (cdecl_)
+          write_h("extern \"C\" { %s%s %s; }\n", rtype, star, name());
+        else
+          write_h("%s%s %s;\n", rtype, star, name());
+      } else {
+        if (havechildren)
+          write_c("static ");
       }
-      else write_c("static ");
-
+      
       // write everything but the default parameters (if any)
       char s[1024], *sptr;
       char *nptr;
       int skips=0,skipc=0;
       int nc=0,plevel=0;
       for (sptr=s,nptr=(char*)name(); *nptr; nc++,nptr++) {
-	if (!skips && *nptr=='(') plevel++;
-	else if (!skips && *nptr==')') plevel--;
-	if ( *nptr=='"' &&  !(nc &&  *(nptr-1)=='\\') ) 
-	  skips = skips ? 0 : 1;
-	else if(!skips && *nptr=='\'' &&  !(nc &&  *(nptr-1)=='\\'))
-	  skipc = skipc ? 0 : 1;
-	if(!skips && !skipc && plevel==1 && *nptr =='=' && 
-	   !(nc && *(nptr-1)=='\'') ) // ignore '=' case 
-	  while(*++nptr  && (skips || skipc || (*nptr!=',' && *nptr!=')' || plevel!=1) )) {
-	    if ( *nptr=='"' &&  *(nptr-1)!='\\' ) 
-	      skips = skips ? 0 : 1;
-	    else if(!skips && *nptr=='\'' &&  *(nptr-1)!='\\')
-	      skipc = skipc ? 0 : 1;
-	    if (!skips && !skipc && *nptr=='(') plevel++;
-	    else if (!skips && *nptr==')') plevel--;
-	  }
-
-	if (sptr < (s + sizeof(s) - 1))	*sptr++ = *nptr;
+        if (!skips && *nptr=='(') plevel++;
+        else if (!skips && *nptr==')') plevel--;
+        if ( *nptr=='"' &&  !(nc &&  *(nptr-1)=='\\') ) 
+          skips = skips ? 0 : 1;
+        else if(!skips && *nptr=='\'' &&  !(nc &&  *(nptr-1)=='\\'))
+          skipc = skipc ? 0 : 1;
+        if(!skips && !skipc && plevel==1 && *nptr =='=' && 
+           !(nc && *(nptr-1)=='\'') ) // ignore '=' case 
+          while(*++nptr  && (skips || skipc || ( (*nptr!=',' && *nptr!=')') || plevel!=1) )) {
+            if ( *nptr=='"' &&  *(nptr-1)!='\\' ) 
+              skips = skips ? 0 : 1;
+            else if(!skips && *nptr=='\'' &&  *(nptr-1)!='\\')
+              skipc = skipc ? 0 : 1;
+            if (!skips && !skipc && *nptr=='(') plevel++;
+            else if (!skips && *nptr==')') plevel--;
+          }
+        
+        if (sptr < (s + sizeof(s) - 1))	*sptr++ = *nptr;
       }
       *sptr = '\0';
- 
-      write_c("%s%s %s {\n", rtype, star, s);
+      
+      if (havechildren)
+        write_c("%s%s %s {\n", rtype, star, s);
     }
   }
-
+  
   if (havewidgets && !child->name()) write_c("  %s* w;\n", subclassname(child));
   indentation += 2;
 }
@@ -366,16 +399,20 @@ void Fl_Function_Type::write_code1() {
 void Fl_Function_Type::write_code2() {
   Fl_Type *child;
   const char *var = "w";
-  for (child = next; child && child->level > level; child = child->next)
+  char havechildren = 0;
+  for (child = next; child && child->level > level; child = child->next) {
+    havechildren = 1;
     if (child->is_window() && child->name()) var = child->name();
-
+  }
+  
   if (ismain()) {
     if (havewidgets) write_c("  %s->show(argc, argv);\n", var);
-    write_c("  return Fl::run();\n");
+    if (havechildren) write_c("  return Fl::run();\n");
   } else if (havewidgets && !constructor && !return_type) {
     write_c("  return %s;\n", var);
   }
-  write_c("}\n");
+  if (havechildren)
+    write_c("}\n");
   indentation = 0;
 }
 
@@ -412,7 +449,7 @@ void Fl_Code_Type::open() {
   code_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == code_panel_cancel) goto BREAK2;
@@ -425,7 +462,7 @@ void Fl_Code_Type::open() {
     free(c);
     break;
   }
- BREAK2:
+BREAK2:
   code_panel->hide();
 }
 
@@ -434,7 +471,15 @@ Fl_Code_Type Fl_Code_type;
 void Fl_Code_Type::write_code1() {
   const char* c = name();
   if (!c) return;
-  write_c("%s%s\n", indent(), c);
+  const char *pch;
+  const char *ind = indent();
+  while( (pch=strchr(c,'\n')) )
+  {
+    int line_len = pch - c;
+    write_c("%s%.*s\n", ind, line_len, c);
+    c = pch+1;
+  }
+  write_c("%s%s\n", ind, c);
 }
 
 void Fl_Code_Type::write_code2() {}
@@ -479,7 +524,7 @@ void Fl_CodeBlock_Type::open() {
   codeblock_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == codeblock_panel_cancel) goto BREAK2;
@@ -494,7 +539,7 @@ void Fl_CodeBlock_Type::open() {
     storestring(c, after);
     break;
   }
- BREAK2:
+BREAK2:
   codeblock_panel->hide();
 }
 
@@ -507,7 +552,7 @@ void Fl_CodeBlock_Type::write_code1() {
 }
 
 void Fl_CodeBlock_Type::write_code2() {
-  indentation += 2;
+  indentation -= 2;
   if (after) write_c("%s} %s\n", indent(), after);
   else write_c("%s}\n", indent());
 }
@@ -516,13 +561,13 @@ void Fl_CodeBlock_Type::write_code2() {
 
 int Fl_Decl_Type::is_public() const 
 {
- Fl_Type *p = parent;
- while (p && !p->is_decl_block()) p = p->parent;
- if(p && p->is_public() && public_)
-   return public_;
- else if(!p)
-   return public_;
- return 0;
+  Fl_Type *p = parent;
+  while (p && !p->is_decl_block()) p = p->parent;
+  if(p && p->is_public() && public_)
+    return public_;
+  else if(!p)
+    return public_;
+  return 0;
 }
 
 Fl_Type *Fl_Decl_Type::make() {
@@ -540,17 +585,25 @@ Fl_Type *Fl_Decl_Type::make() {
 void Fl_Decl_Type::write_properties() {
   Fl_Type::write_properties();
   switch (public_) {
+    case 0: write_string("private"); break;
     case 1: write_string("public"); break;
     case 2: write_string("protected"); break;
   }
-  if (!static_) write_string("global");
+  if (static_) 
+    write_string("local");
+  else
+    write_string("global");
 }
 
 void Fl_Decl_Type::read_property(const char *c) {
   if (!strcmp(c,"public")) {
     public_ = 1;
+  } else if (!strcmp(c,"private")) {
+    public_ = 0;
   } else if (!strcmp(c,"protected")) {
     public_ = 2;
+  } else if (!strcmp(c,"local")) {
+    static_ = 1;
   } else if (!strcmp(c,"global")) {
     static_ = 0;
   } else {
@@ -570,10 +623,12 @@ void Fl_Decl_Type::open() {
     decl_choice->show();
     decl_class_choice->hide();
   }
+  const char *c = comment();
+  decl_comment_input->buffer()->text(c?c:"");
   decl_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == decl_panel_cancel) goto BREAK2;
@@ -600,9 +655,18 @@ void Fl_Decl_Type::open() {
         static_ = ((decl_choice->value()>>1)&1);
       }
     }
+    c = decl_comment_input->buffer()->text();
+    if (c && *c) {
+      if (!comment() || strcmp(c, comment())) redraw_browser();
+      comment(c);
+    } else {
+      if (comment()) redraw_browser();
+      comment(0);
+    }
+    if (c) free((void*)c);
     break;
   }
- BREAK2:
+BREAK2:
   decl_panel->hide();
 }
 
@@ -612,49 +676,58 @@ void Fl_Decl_Type::write_code1() {
   const char* c = name();
   if (!c) return;
   // handle a few keywords differently if inside a class
-  if (is_in_class() && (
-         !strncmp(c,"class",5) && isspace(c[5])
-      || !strncmp(c,"typedef",7) && isspace(c[7])
-      || !strncmp(c,"FL_EXPORT",9) && isspace(c[9])
-      || !strncmp(c,"struct",6) && isspace(c[6])
-      ) ) {
+  if (is_in_class() && (   (!strncmp(c,"class",5) && isspace(c[5]))
+                        || (!strncmp(c,"typedef",7) && isspace(c[7]))
+                        || (!strncmp(c,"FL_EXPORT",9) && isspace(c[9]))
+                        || (!strncmp(c,"struct",6) && isspace(c[6]))
+                        ) ) {
     write_public(public_);
+    write_comment_h("  ");
     write_h("  %s\n", c);
     return;
   }
   // handle putting #include, extern, using or typedef into decl:
-  if (!isalpha(*c) && *c != '~'
-      || !strncmp(c,"extern",6) && isspace(c[6])
-      || !strncmp(c,"class",5) && isspace(c[5])
-      || !strncmp(c,"typedef",7) && isspace(c[7])
-      || !strncmp(c,"using",5) && isspace(c[5])
-      || !strncmp(c,"FL_EXPORT",9) && isspace(c[9])
-//    || !strncmp(c,"struct",6) && isspace(c[6])
+  if (   (!isalpha(*c) && *c != '~')
+      || (!strncmp(c,"extern",6) && isspace(c[6]))
+      || (!strncmp(c,"class",5) && isspace(c[5]))
+      || (!strncmp(c,"typedef",7) && isspace(c[7]))
+      || (!strncmp(c,"using",5) && isspace(c[5]))
+      || (!strncmp(c,"FL_EXPORT",9) && isspace(c[9]))
+      //    || !strncmp(c,"struct",6) && isspace(c[6])
       ) {
-    if (public_)
+    if (public_) {
+      write_comment_h();
       write_h("%s\n", c);
-    else
+    } else {
+      write_comment_c();
       write_c("%s\n", c);
+    }
     return;
   }
   // find the first C++ style comment
   const char* e = c+strlen(c), *csc = c;
   while (csc<e && (csc[0]!='/' || csc[1]!='/')) csc++;
   if (csc!=e) e = csc; // comment found
-  // lose all trailing semicolons so I can add one:
+                       // lose all trailing semicolons so I can add one:
   while (e>c && e[-1]==' ') e--;
   while (e>c && e[-1]==';') e--;
   if (class_name(1)) {
     write_public(public_);
+    write_comment_h("  ");
     write_h("  %.*s; %s\n", (int)(e-c), c, csc);
   } else {
     if (public_) {
       if (static_) 
         write_h("extern ");
+      else
+        write_comment_h();
       write_h("%.*s; %s\n", (int)(e-c), c, csc);
-      if (static_)
+      if (static_) {
+        write_comment_c();
         write_c("%.*s; %s\n", (int)(e-c), c, csc);
+      }
     } else {
+      write_comment_c();
       if (static_) 
         write_c("static ");
       write_c("%.*s; %s\n", (int)(e-c), c, csc);
@@ -666,6 +739,210 @@ void Fl_Decl_Type::write_code2() {}
 
 ////////////////////////////////////////////////////////////////
 
+Fl_Type *Fl_Data_Type::make() {
+  Fl_Type *p = Fl_Type::current;
+  while (p && !p->is_decl_block()) p = p->parent;
+  Fl_Data_Type *o = new Fl_Data_Type();
+  o->public_ = 1;
+  o->static_ = 1;
+  o->filename_ = 0;
+  o->name("myBinaryData");
+  o->add(p);
+  o->factory = this;
+  return o;
+}
+
+void Fl_Data_Type::write_properties() {
+  Fl_Decl_Type::write_properties();
+  if (filename_) {
+    write_string("filename");
+    write_word(filename_);
+  }
+}
+
+void Fl_Data_Type::read_property(const char *c) {
+  if (!strcmp(c,"filename")) {
+    storestring(read_word(), filename_, 1);
+  } else {
+    Fl_Decl_Type::read_property(c);
+  }
+}
+
+void Fl_Data_Type::open() {
+  if (!data_panel) make_data_panel();
+  data_input->static_value(name());
+  if (is_in_class()) {
+    data_class_choice->value(public_);
+    data_class_choice->show();
+    data_choice->hide();
+  } else {
+    data_choice->value((public_&1)|((static_&1)<<1));
+    data_choice->show();
+    data_class_choice->hide();
+  }
+  data_filename->value(filename_?filename_:"");
+  const char *c = comment();
+  data_comment_input->buffer()->text(c?c:"");
+  data_panel->show();
+  const char* message = 0;
+  for (;;) { // repeat as long as there are errors
+    if (message) fl_alert("%s", message);
+    for (;;) {
+      Fl_Widget* w = Fl::readqueue();
+      if (w == data_panel_cancel) goto BREAK2;
+      else if (w == data_panel_ok) break;
+      else if (w == data_filebrowser) {
+        goto_source_dir();
+        const char *fn = fl_file_chooser("Load Binary Data", 0L, data_filename->value(), 1);
+        leave_source_dir();
+        if (fn) {
+          if (strcmp(fn, data_filename->value()))
+            set_modflag(1); 
+          data_filename->value(fn);
+        }
+      }
+      else if (!w) Fl::wait();
+    }
+    // store the variable name:
+    const char*c = data_input->value();
+    char *s = strdup(c), *p = s, *q, *n;
+    for (;;++p) {
+      if (!isspace((unsigned char)(*p))) break;
+    }
+    n = p;
+    if ( (!isalpha((unsigned char)(*p))) && ((*p)!='_') && ((*p)!=':') ) goto OOPS;
+    ++p;
+    for (;;++p) {
+      if ( (!isalnum((unsigned char)(*p))) && ((*p)!='_') && ((*p)!=':') ) break;
+    }
+    q = p;
+    for (;;++q) {
+      if (!*q) break;
+      if (!isspace((unsigned char)(*q))) goto OOPS;
+    }		
+    if (n==q) {
+    OOPS: message = "variable name must be a C identifier";
+      free((void*)s);
+      continue;
+    }
+    *p = 0;
+    name(n);
+    free(s);
+    // store flags
+    if (is_in_class()) {
+      if (public_!=data_class_choice->value()) {
+        set_modflag(1);
+        public_ = data_class_choice->value();
+      }
+    } else {
+      if (public_!=(data_choice->value()&1)) {
+        set_modflag(1);
+        public_ = (data_choice->value()&1);
+      }
+      if (static_!=((data_choice->value()>>1)&1)) {
+        set_modflag(1);
+        static_ = ((data_choice->value()>>1)&1);
+      }
+    }
+    // store the filename
+    c = data_filename->value();
+    if (filename_ && strcmp(filename_, data_filename->value()))
+      set_modflag(1); 
+    else if (!filename_ && *c)
+      set_modflag(1);
+    if (filename_) { free((void*)filename_); filename_ = 0L; }
+    if (c && *c) filename_ = strdup(c);
+    // store the comment
+    c = data_comment_input->buffer()->text();
+    if (c && *c) {
+      if (!comment() || strcmp(c, comment())) redraw_browser();
+      comment(c);
+    } else {
+      if (comment()) redraw_browser();
+      comment(0);
+    }
+    if (c) free((void*)c);
+    break;
+  }
+BREAK2:
+  data_panel->hide();
+}
+
+Fl_Data_Type Fl_Data_type;
+
+void Fl_Data_Type::write_code1() {
+  const char *message = 0;
+  const char *c = name();
+  if (!c) return;
+  const char *fn = filename_;
+  char *data = 0;
+  int nData = -1;
+  // path should be set correctly already
+  if (filename_ && !write_sourceview) {
+    FILE *f = fl_fopen(filename_, "rb");
+    if (!f) {
+      message = "Can't include binary file. Can't open";
+    } else {
+      fseek(f, 0, SEEK_END);
+      nData = ftell(f);
+      fseek(f, 0, SEEK_SET);
+      if (nData) {
+        data = (char*)calloc(nData, 1);
+        if (fread(data, nData, 1, f)==0) { /* use default */ }
+      }
+      fclose(f);
+    }
+  } else {
+    fn = "<no filename>";
+  }
+  if (is_in_class()) {
+    write_public(public_);
+    write_comment_h("  ");
+    write_h("  static unsigned char %s[%d];\n", c, nData);
+    write_c("unsigned char %s::%s[%d] = /* binary data included from %s */\n", class_name(1), c, nData, fn);
+    if (message) write_c("#error %s %s\n", message, fn);
+    write_cdata(data, nData);
+    write_c(";\n");
+  } else {
+    // the "header only" option does not apply here!
+    if (public_) {
+      if (static_) {
+        write_h("extern unsigned char %s[%d];\n", c, nData);
+        write_comment_c();
+        write_c("unsigned char %s[%d] = /* binary data included from %s */\n", c, nData, fn);
+        if (message) write_c("#error %s %s\n", message, fn);
+        write_cdata(data, nData);
+        write_c(";\n");
+      } else {
+        write_comment_h();
+        write_h("#error Unsupported declaration loading binary data %s\n", fn);
+        write_h("unsigned char %s[3] = { 1, 2, 3 };\n", c);
+      }
+    } else {
+      write_comment_c();
+      if (static_) 
+        write_c("static ");
+      write_c("unsigned char %s[%d] = /* binary data included from %s */\n", c, nData, fn);
+      if (message) write_c("#error %s %s\n", message, fn);
+      write_cdata(data, nData);
+      write_c(";\n");
+    }
+  }
+  // if we are in interactive mode, we pop up a warning dialog 
+  // giving the error: (compile_only && !write_sourceview)
+  if (message && !write_sourceview) {
+    if (compile_only)
+      fprintf(stderr, "FLUID ERROR: %s %s\n", message, fn);
+    else
+      fl_alert("%s\n%s\n", message, fn);
+  }
+  if (data) free(data);
+}
+
+void Fl_Data_Type::write_code2() {}
+
+////////////////////////////////////////////////////////////////
+
 int Fl_DeclBlock_Type::is_public() const {return public_;}
 
 Fl_Type *Fl_DeclBlock_Type::make() {
@@ -710,7 +987,7 @@ void Fl_DeclBlock_Type::open() {
   declblock_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == declblock_panel_cancel) goto BREAK2;
@@ -734,7 +1011,7 @@ void Fl_DeclBlock_Type::open() {
     }
     break;
   }
- BREAK2:
+BREAK2:
   declblock_panel->hide();
 }
 
@@ -743,14 +1020,14 @@ Fl_DeclBlock_Type Fl_DeclBlock_type;
 void Fl_DeclBlock_Type::write_code1() {
   const char* c = name();
   if (public_)
-     write_h("%s\n", c);
+    write_h("%s\n", c);
   write_c("%s\n", c);
 }
 
 void Fl_DeclBlock_Type::write_code2() {
   const char* c = after;
   if (public_)
-     write_h("%s\n", c);
+    write_h("%s\n", c);
   write_c("%s\n", c);
 }
 
@@ -830,10 +1107,10 @@ void Fl_Comment_Type::open() {
   comment_in_header->value(in_h_);
   comment_panel->show();
   const char* message = 0;
-  char itempath[256]; itempath[0] = 0;
+  char itempath[FL_PATH_MAX]; itempath[0] = 0;
   int last_selected_item = 0;
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == comment_panel_cancel) goto BREAK2;
@@ -842,9 +1119,9 @@ void Fl_Comment_Type::open() {
         if (comment_predefined->value()==1) {
           // add the current comment to the database
           const char *xname = fl_input(
-            "Please enter a name to reference the current\ncomment in your database.\n\n"
-            "Use forward slashes '/' to create submenus.", 
-            "My Comment");
+                                       "Please enter a name to reference the current\ncomment in your database.\n\n"
+                                       "Use forward slashes '/' to create submenus.", 
+                                       "My Comment");
           if (xname) {
             char *name = strdup(xname);
             for (char*s=name;*s;s++) if (*s==':') *s = ';';
@@ -920,7 +1197,7 @@ void Fl_Comment_Type::open() {
     if (mod) set_modflag(1);
     break;
   }
- BREAK2:
+BREAK2:
   title_buf[0] = 0;
   comment_panel->hide();
 }
@@ -1073,7 +1350,7 @@ void Fl_Class_Type::read_property(const char *c) {
 
 void Fl_Class_Type::open() {
   if (!class_panel) make_class_panel();
-  char fullname[1024]="";
+  char fullname[FL_PATH_MAX]="";
   if (prefix() && strlen(prefix())) 
     sprintf(fullname,"%s %s",prefix(),name());
   else 
@@ -1081,13 +1358,15 @@ void Fl_Class_Type::open() {
   c_name_input->static_value(fullname);
   c_subclass_input->static_value(subclass_of);
   c_public_button->value(public_);
+  const char *c = comment();
+  c_comment_input->buffer()->text(c?c:"");
   class_panel->show();
   const char* message = 0;
-
+  
   char *na=0,*pr=0,*p=0; // name and prefix substrings
-
+  
   for (;;) { // repeat as long as there are errors
-    if (message) fl_alert(message);
+    if (message) fl_alert("%s", message);
     for (;;) {
       Fl_Widget* w = Fl::readqueue();
       if (w == c_panel_cancel) goto BREAK2;
@@ -1103,7 +1382,7 @@ void Fl_Class_Type::open() {
     if (p<s) goto OOPS;
     while (p>=s && is_id(*p)) p--;
     if ( (p<s && !is_id(*(p+1))) || !*(p+1) ) {
-      OOPS: message = "class name must be C++ identifier";
+    OOPS: message = "class name must be C++ identifier";
       free((void*)s);
       continue;
     }
@@ -1124,9 +1403,18 @@ void Fl_Class_Type::open() {
       public_ = c_public_button->value();
       set_modflag(1);
     }
+    c = c_comment_input->buffer()->text();
+    if (c && *c) {
+      if (!comment() || strcmp(c, comment())) redraw_browser();
+      comment(c);
+    } else {
+      if (comment()) redraw_browser();
+      comment(0);
+    }
+    if (c) free((void*)c);
     break;
   }
- BREAK2:
+BREAK2:
   class_panel->hide();
 }
 
@@ -1151,10 +1439,12 @@ void Fl_Class_Type::write_code1() {
   parent_class = current_class;
   current_class = this;
   write_public_state = 0;
+  write_h("\n");
+  write_comment_h();
   if (prefix() && strlen(prefix()))
-      write_h("\nclass %s %s ", prefix(), name());
+    write_h("class %s %s ", prefix(), name());
   else
-      write_h("\nclass %s ", name());
+    write_h("class %s ", name());
   if (subclass_of) write_h(": %s ", subclass_of);
   write_h("{\n");
 }
@@ -1180,5 +1470,5 @@ int Fl_Class_Type::has_function(const char *rtype, const char *sig) const {
 }
 
 //
-// End of "$Id: Fl_Function_Type.cxx 6013 2008-01-04 22:18:40Z matt $".
+// End of "$Id: Fl_Function_Type.cxx 8786 2011-06-07 11:41:36Z manolo $".
 //
