@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Widget_Type.cxx 6801 2009-06-28 22:09:25Z matt $"
+// "$Id: Fl_Widget_Type.cxx 8172 2011-01-03 08:28:38Z matt $"
 //
 // Widget type code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -27,6 +27,7 @@
 
 #include <FL/Fl.H>
 #include <FL/Fl_Group.H>
+#include <FL/Fl_Table.H>
 #include <FL/Fl_Input.H>
 #include "Fl_Widget_Type.h"
 #include "alignment_panel.h"
@@ -67,6 +68,10 @@ const char* subclassname(Fl_Type* l) {
     if (c) return c;
     if (l->is_class()) return "Fl_Group";
     if (p->o->type() == FL_WINDOW+1) return "Fl_Double_Window";
+    if (strcmp(p->type_name(), "Fl_Input") == 0) {
+      if (p->o->type() == FL_FLOAT_INPUT) return "Fl_Float_Input";
+      if (p->o->type() == FL_INT_INPUT) return "Fl_Int_Input";
+    }
   }
   return l->type_name();
 }
@@ -198,7 +203,8 @@ void Fl_Widget_Type::setlabel(const char *n) {
 }
 
 Fl_Widget_Type::Fl_Widget_Type() {
-  for (int n=0; n<NUM_EXTRA_CODE; n++) {extra_code_[n] = 0; subclass_ = 0;}
+  for (int n=0; n<NUM_EXTRA_CODE; n++) {extra_code_[n] = 0; }
+  subclass_ = 0;
   hotspot_ = 0;
   tooltip_ = 0;
   image_name_ = 0;
@@ -216,6 +222,13 @@ Fl_Widget_Type::~Fl_Widget_Type() {
     if (o->parent()) ((Fl_Group*)o->parent())->remove(*o);
     delete o;
   }
+  if (subclass_) free((void*)subclass_);
+  if (tooltip_) free((void*)tooltip_);
+  if (image_name_) free((void*)image_name_);
+  if (inactive_name_) free((void*)inactive_name_);
+  for (int n=0; n<NUM_EXTRA_CODE; n++) {
+    if (extra_code_[n]) free((void*) extra_code_[n]);
+  }
 }
 
 void Fl_Widget_Type::extra_code(int m,const char *n) {
@@ -260,7 +273,7 @@ void Fl_Widget_Type::redraw() {
 Fl_Type *sort(Fl_Type *parent) {
   Fl_Type *f,*n=0;
   for (f = parent ? parent->next : Fl_Type::first; ; f = n) {
-    if (!f || parent && f->level <= parent->level) return f;
+    if (!f || (parent && f->level <= parent->level)) return f;
     n = sort(f);
     if (!f->selected || (!f->is_widget() || f->is_menu_item())) continue;
     Fl_Widget* fw = ((Fl_Widget_Type*)f)->o;
@@ -291,6 +304,7 @@ static Fl_Window *the_panel;
 // initialized parts of the widget that are nyi by fluid.
 
 Fl_Widget_Type *current_widget; // one of the selected ones
+void* const LOAD = (void *)"LOAD"; // "magic" pointer to indicate that we need to load values into the dialog
 static int numselected; // number selected
 static int haderror;
 
@@ -1133,7 +1147,7 @@ static Fl_Menu_Item alignmenu[] = {
 {0}};
 
 void align_cb(Fl_Button* i, void *v) {
-  int b = int(long(i->user_data()));
+  Fl_Align b = Fl_Align(fl_uintptr_t(i->user_data()));
   if (v == LOAD) {
     if (current_widget->is_menu_item()) {i->deactivate(); return;} else i->activate();
     i->value(current_widget->o->align() & b);
@@ -1142,8 +1156,8 @@ void align_cb(Fl_Button* i, void *v) {
     for (Fl_Type *o = Fl_Type::first; o; o = o->next) {
       if (o->selected && o->is_widget()) {
 	Fl_Widget_Type* q = (Fl_Widget_Type*)o;
-	int x = q->o->align();
-	int y;
+	Fl_Align x = q->o->align();
+	Fl_Align y;
 	if (i->value()) {
 	  y = x | b;
 	  if (b == FL_ALIGN_LEFT || b == FL_ALIGN_TOP) {
@@ -1170,6 +1184,64 @@ void align_cb(Fl_Button* i, void *v) {
   }
 }
 
+void align_position_cb(Fl_Choice *i, void *v) {
+  if (v == LOAD) {
+    if (current_widget->is_menu_item()) {i->deactivate(); return;} else i->activate();
+    Fl_Menu_Item *mi = (Fl_Menu_Item*)i->menu();
+    Fl_Align b = current_widget->o->align() & FL_ALIGN_POSITION_MASK;
+    for (;mi->text;mi++) {
+      if ((Fl_Align)(mi->argument())==b)
+        i->value(mi);
+    }
+  } else {
+    const Fl_Menu_Item *mi = i->menu() + i->value();
+    Fl_Align b = Fl_Align(fl_uintptr_t(mi->user_data()));
+    int mod = 0;
+    for (Fl_Type *o = Fl_Type::first; o; o = o->next) {
+      if (o->selected && o->is_widget()) {
+	Fl_Widget_Type* q = (Fl_Widget_Type*)o;
+	Fl_Align x = q->o->align();
+	Fl_Align y = (x & ~FL_ALIGN_POSITION_MASK) | b;
+	if (x != y) {
+          q->o->align(y);
+	  q->redraw();
+	  mod = 1;
+	}
+      }
+    }
+    if (mod) set_modflag(1);
+  }
+}
+
+void align_text_image_cb(Fl_Choice *i, void *v) {
+  if (v == LOAD) {
+    if (current_widget->is_menu_item()) {i->deactivate(); return;} else i->activate();
+    Fl_Menu_Item *mi = (Fl_Menu_Item*)i->menu();
+    Fl_Align b = current_widget->o->align() & FL_ALIGN_IMAGE_MASK;
+    for (;mi->text;mi++) {
+      if ((Fl_Align)(mi->argument())==b)
+        i->value(mi);
+    }
+  } else {
+    const Fl_Menu_Item *mi = i->menu() + i->value();
+    Fl_Align b = Fl_Align(fl_uintptr_t(mi->user_data()));
+    int mod = 0;
+    for (Fl_Type *o = Fl_Type::first; o; o = o->next) {
+      if (o->selected && o->is_widget()) {
+	Fl_Widget_Type* q = (Fl_Widget_Type*)o;
+	Fl_Align x = q->o->align();
+	Fl_Align y = (x & ~FL_ALIGN_IMAGE_MASK) | b;
+	if (x != y) {
+          q->o->align(y);
+	  q->redraw();
+	  mod = 1;
+	}
+      }
+    }
+    if (mod) set_modflag(1);
+  }
+}
+
 ////////////////////////////////////////////////////////////////
 
 void callback_cb(CodeEditor* i, void *v) {
@@ -1244,7 +1316,7 @@ void user_data_type_cb(Fl_Input *i, void *v) {
 // "v_attributes" let user type in random code for attribute settings:
 
 void v_input_cb(Fl_Input* i, void* v) {
-  int n = int(long(i->user_data()));
+  int n = fl_intptr_t(i->user_data());
   if (v == LOAD) {
     i->static_value(current_widget->extra_code(n));
   } else {
@@ -1728,6 +1800,7 @@ void live_mode_cb(Fl_Button*o,void *) {
   }
   if (o->value()) {
     if (numselected == 1) {
+      Fl_Group::current(0L);
       live_widget = current_widget->enter_live_mode(1);
       if (live_widget) {
         live_type = current_widget;
@@ -1745,8 +1818,9 @@ void live_mode_cb(Fl_Button*o,void *) {
         Fl_Button *btn = new Fl_Button(10, h+20, 100, 25, "Exit Live Mode");
         btn->labelsize(12);
         btn->callback(leave_live_mode_cb);
-        live_widget->position(10, 10);
+        rsz->end();
         live_window->add(live_widget);
+        live_widget->position(10, 10);
         live_window->resizable(live_widget);
         live_window->set_modal(); // block all other UI
         live_window->callback(leave_live_mode_cb);
@@ -1808,6 +1882,7 @@ void Fl_Widget_Type::open() {
 Fl_Type *Fl_Type::current;
 
 extern void redraw_overlays();
+extern void check_redraw_corresponding_parent(Fl_Type*);
 extern void redraw_browser();
 extern void update_sourceview_position();
 
@@ -1839,6 +1914,7 @@ void selection_changed(Fl_Type *p) {
   }
   if (!p || !p->selected) p = q;
   Fl_Type::current = p;
+  check_redraw_corresponding_parent(p);
   redraw_overlays();
   // load the panel with the new settings:
   load_panel();
@@ -1878,7 +1954,7 @@ const char *array_name(Fl_Widget_Type *o) {
     if (!e) continue;
     if (strncmp(c,e,d-c)) continue;
     int n1 = atoi(e+(d-c)+1);
-    if (n1 > num || n1==num && sawthis) return 0;
+    if (n1 > num || (n1==num && sawthis)) return 0;
   }
   static char buffer[128];
   // MRS: we want strncpy() here...
@@ -2124,8 +2200,7 @@ void Fl_Widget_Type::write_color(const char* field, Fl_Color color) {
   }
   const char *var = is_class() ? "this" : name() ? name() : "o";
   if (color_name) {
-    write_c("%s%s->%s((Fl_Color)%s);\n", indent(), var, field, color_name);
-    // we still must cast ^^^^^ here because a few widgets have the wrong argument type. Ouch!
+    write_c("%s%s->%s(%s);\n", indent(), var, field, color_name);
   } else {
     write_c("%s%s->%s((Fl_Color)%d);\n", indent(), var, field, color);
   }
@@ -2163,19 +2238,26 @@ void Fl_Widget_Type::write_widget_code() {
     write_c("%s%s->type(%d);\n", indent(), var, o->type());
   if (o->box() != tplate->box() || subclass())
     write_c("%s%s->box(FL_%s);\n", indent(), var, boxname(o->box()));
+
+  // write shortcut command if needed
+  int shortcut = 0;
+  if (is_button()) shortcut = ((Fl_Button*)o)->shortcut();
+  else if (is_input()) shortcut = ((Fl_Input_*)o)->shortcut();
+  else if (is_value_input()) shortcut = ((Fl_Value_Input*)o)->shortcut();
+  else if (is_text_display()) shortcut = ((Fl_Text_Display*)o)->shortcut();
+  if (shortcut) {
+    if (use_FL_COMMAND && (shortcut & (FL_CTRL|FL_META))) {
+      write_c("%s%s->shortcut(FL_COMMAND|0x%x);\n", indent(), var, shortcut & ~(FL_CTRL|FL_META));
+    } else {
+      write_c("%s%s->shortcut(0x%x);\n", indent(), var, shortcut);
+    }
+  }
+
   if (is_button()) {
     Fl_Button* b = (Fl_Button*)o;
     if (b->down_box()) write_c("%s%s->down_box(FL_%s);\n", indent(), var,
 			       boxname(b->down_box()));
     if (b->value()) write_c("%s%s->value(1);\n", indent(), var);
-    if (b->shortcut()) {
-			int s = b->shortcut();
-			if (use_FL_COMMAND && (s & (FL_CTRL|FL_META))) {
-		    write_c("%s%s->shortcut(FL_COMMAND|0x%x);\n", indent(), var, s & ~(FL_CTRL|FL_META));
-			} else {
-	      write_c("%s%s->shortcut(0x%x);\n", indent(), var, s);
-			}
-		}
   } else if (!strcmp(type_name(), "Fl_Input_Choice")) {
     Fl_Input_Choice* b = (Fl_Input_Choice*)o;
     if (b->down_box()) write_c("%s%s->down_box(FL_%s);\n", indent(), var,
@@ -2254,10 +2336,10 @@ void Fl_Widget_Type::write_widget_code() {
   }
   if (o->align() != tplate->align() || subclass()) {
     int i = o->align();
-    write_c("%s%s->align(%s", indent(), var,
+    write_c("%s%s->align(Fl_Align(%s", indent(), var,
 	    item_name(alignmenu, i & ~FL_ALIGN_INSIDE));
     if (i & FL_ALIGN_INSIDE) write_c("|FL_ALIGN_INSIDE");
-    write_c(");\n");
+    write_c("));\n");
   }
   // avoid the unsupported combination of flegs when user sets 
   // "when" to "FL_WHEN_NEVER", but keeps the "no change" set. 
@@ -2334,6 +2416,18 @@ void Fl_Widget_Type::write_properties() {
   }
   if (o->box() != tplate->box()) {
     write_string("box"); write_word(boxname(o->box()));}
+  if (is_input()) {
+    Fl_Input_* b = (Fl_Input_*)o;
+    if (b->shortcut()) write_string("shortcut 0x%x", b->shortcut());
+  }
+  if (is_value_input()) {
+    Fl_Value_Input* b = (Fl_Value_Input*)o;
+    if (b->shortcut()) write_string("shortcut 0x%x", b->shortcut());
+  }
+  if (is_text_display()) {
+    Fl_Text_Display* b = (Fl_Text_Display*)o;
+    if (b->shortcut()) write_string("shortcut 0x%x", b->shortcut());
+  }
   if (is_button()) {
     Fl_Button* b = (Fl_Button*)o;
     if (b->down_box()) {
@@ -2411,7 +2505,7 @@ void Fl_Widget_Type::write_properties() {
 }
 
 int pasteoffset;
-
+extern double read_version;
 void Fl_Widget_Type::read_property(const char *c) {
   int x,y,w,h; Fl_Font f; int s; Fl_Color cc;
   if (!strcmp(c,"private")) {
@@ -2422,6 +2516,11 @@ void Fl_Widget_Type::read_property(const char *c) {
     if (sscanf(read_word(),"%d %d %d %d",&x,&y,&w,&h) == 4) {
       x += pasteoffset;
       y += pasteoffset;
+      // FIXME temporary change!
+      if (read_version>=2.0 && o->parent() && o->parent()!=o->window()) {
+        x += o->parent()->x();
+        y += o->parent()->y();
+      }
       o->resize(x,y,w,h);
     }
   } else if (!strcmp(c,"tooltip")) {
@@ -2463,12 +2562,18 @@ void Fl_Widget_Type::read_property(const char *c) {
     const char* value = read_word();
     ((Fl_Button*)o)->value(atoi(value));
   } else if (!strcmp(c,"color")) {
-    int n = sscanf(read_word(),"%d %d",&x,&y);
-    if (n == 2) { // back compatability...
-      if (x != 47) o->color(x);
-      o->selection_color(y);
-    } else {
+    const char *cw = read_word();
+    if (cw[0]=='0' && cw[1]=='x') {
+      sscanf(cw,"0x%x",&x);
       o->color(x);
+    } else {
+      int n = sscanf(cw,"%d %d",&x,&y);
+      if (n == 2) { // back compatibility...
+        if (x != 47) o->color(x);
+        o->selection_color(y);
+      } else {
+        o->color(x);
+      }
     }
   } else if (!strcmp(c,"selection_color")) {
     if (sscanf(read_word(),"%d",&x)) o->selection_color(x);
@@ -2523,15 +2628,22 @@ void Fl_Widget_Type::read_property(const char *c) {
     hotspot(1);
   } else if (!strcmp(c,"class")) {
     subclass(read_word());
-  } else if (is_button() && !strcmp(c,"shortcut")) {
-    ((Fl_Button*)o)->shortcut(strtol(read_word(),0,0));
+  } else if (!strcmp(c,"shortcut")) {
+    int shortcut = strtol(read_word(),0,0);
+    if (is_button()) ((Fl_Button*)o)->shortcut(shortcut);
+    else if (is_input()) ((Fl_Input_*)o)->shortcut(shortcut);
+    else if (is_value_input()) ((Fl_Value_Input*)o)->shortcut(shortcut);
+    else if (is_text_display()) ((Fl_Text_Display*)o)->shortcut(shortcut);
   } else {
     if (!strncmp(c,"code",4)) {
       int n = atoi(c+4);
       if (n >= 0 && n <= NUM_EXTRA_CODE) {
-	extra_code(n,read_word());
-	return;
+        extra_code(n,read_word());
+        return;
       }
+    } else if (!strcmp(c,"extra_code")) {
+      extra_code(0,read_word());
+      return;
     }
     Fl_Type::read_property(c);
   }
@@ -2695,6 +2807,24 @@ void Fl_Widget_Type::copy_properties() {
     d->value(s->value());
   }
 
+  // copy all attributes specific to widgets derived from Fl_Input_
+  if (is_input()) {
+    Fl_Input_* d = (Fl_Input_*)live_widget, *s = (Fl_Input_*)o;
+    d->shortcut(s->shortcut());
+  }
+
+  // copy all attributes specific to widgets derived from Fl_Value_Input
+  if (is_value_input()) {
+    Fl_Value_Input* d = (Fl_Value_Input*)live_widget, *s = (Fl_Value_Input*)o;
+    d->shortcut(s->shortcut());
+  }
+
+  // copy all attributes specific to widgets derived from Fl_Text_Display
+  if (is_text_display()) {
+    Fl_Text_Display* d = (Fl_Text_Display*)live_widget, *s = (Fl_Text_Display*)o;
+    d->shortcut(s->shortcut());
+  }
+
   // copy all attributes specific to Fl_Valuator and derived classes
   if (is_valuator()) {
     Fl_Valuator* d = (Fl_Valuator*)live_widget, *s = (Fl_Valuator*)o;
@@ -2741,5 +2871,5 @@ void Fl_Pack_Type::copy_properties()
 }
 
 //
-// End of "$Id: Fl_Widget_Type.cxx 6801 2009-06-28 22:09:25Z matt $".
+// End of "$Id: Fl_Widget_Type.cxx 8172 2011-01-03 08:28:38Z matt $".
 //
