@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Window_Type.cxx 6098 2008-04-12 18:04:09Z matt $"
+// "$Id: Fl_Window_Type.cxx 8089 2010-12-20 22:19:24Z matt $"
 //
 // Window type code for the Fast Light Tool Kit (FLTK).
 //
@@ -7,7 +7,7 @@
 // for interacting with the overlay, which allows the user to
 // select, move, and resize the children widgets.
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -84,7 +84,7 @@ void guides_cb(Fl_Check_Button *i, long) {
   }
 }
 
-void grid_cb(Fl_Input *i, long v) {
+void grid_cb(Fl_Int_Input *i, long v) {
   int n = atoi(i->value());
   if (n < 0) n = 0;
   switch (v) {
@@ -165,16 +165,23 @@ void i18n_type_cb(Fl_Choice *c, void *) {
 
 void i18n_text_cb(Fl_Input *i, void *) {
   undo_checkpoint();
-
+  
   if (i == i18n_function_input)
     i18n_function = i->value();
   else if (i == i18n_file_input)
     i18n_file = i->value();
-  else if (i == i18n_set_input)
-    i18n_set = i->value();
   else if (i == i18n_include_input)
     i18n_include = i->value();
+  
+  set_modflag(1);
+}
 
+void i18n_int_cb(Fl_Int_Input *i, void *) {
+  undo_checkpoint();
+  
+  if (i == i18n_set_input)
+    i18n_set = i->value();
+  
   set_modflag(1);
 }
 
@@ -238,6 +245,11 @@ void show_settings_cb(Fl_Widget *, void *) {
   settings_window->show();
 }
 
+void show_global_settings_cb(Fl_Widget *, void *) {
+  global_settings_window->hotspot(global_settings_window);
+  show_global_settings_window();
+}
+
 void header_input_cb(Fl_Input* i, void*) {
   if (header_file_name && strcmp(header_file_name, i->value()))
     set_modflag(1);
@@ -306,7 +318,8 @@ extern Fl_Window *main_window;
 // Read an image of the overlay window
 uchar *Overlay_Window::read_image(int &ww, int &hh) {
   // Create an off-screen buffer for the window...
-  main_window->make_current();
+  //main_window->make_current();
+  make_current();
 
   ww = w();
   hh = h();
@@ -329,7 +342,7 @@ uchar *Overlay_Window::read_image(int &ww, int &hh) {
 
   // Cleanup and return...
   fl_delete_offscreen(offscreen);
-
+  main_window->make_current();
   return pixels;
 }
 
@@ -572,10 +585,34 @@ void Fl_Window_Type::newposition(Fl_Widget_Type *myo,int &X,int &Y,int &R,int &T
     R += dx;
     T += dy;
   } else {
-    if (drag&LEFT) if (X==bx) X += dx; else if (X<bx+dx) X = bx+dx;
-    if (drag&TOP) if (Y==by) Y += dy; else if (Y<by+dy) Y = by+dy;
-    if (drag&RIGHT) if (R==br) R += dx; else if (R>br+dx) R = br+dx;
-    if (drag&BOTTOM) if (T==bt) T += dy; else if (T>bt+dx) T = bt+dx;
+    if (drag&LEFT) {
+      if (X==bx) {
+        X += dx; 
+      } else {
+        if (X<bx+dx) X = bx+dx;
+      }
+    }
+    if (drag&TOP) {
+      if (Y==by) {
+        Y += dy;
+      } else {
+        if (Y<by+dy) Y = by+dy;
+      }
+    }
+    if (drag&RIGHT) {
+      if (R==br) {
+        R += dx; 
+      } else {
+        if (R>br+dx) R = br+dx;
+      }
+    }
+    if (drag&BOTTOM) {
+      if (T==bt) {
+        T += dy; 
+      } else {
+        if (T>bt+dx) T = bt+dx;
+      }
+    }
   }
   if (R<X) {int n = X; X = R; R = n;}
   if (T<Y) {int n = Y; Y = T; T = n;}
@@ -1064,6 +1101,22 @@ void Fl_Window_Type::fix_overlay() {
   ((Overlay_Window *)(this->o))->redraw_overlay();
 }
 
+// check if we must redraw any parent of tabs/wizard type
+void check_redraw_corresponding_parent(Fl_Type *s) {
+    Fl_Widget_Type * prev_parent = 0;
+    if( !s || !s->selected || !s->is_widget()) return;
+    for (Fl_Type *i=s; i && i->parent; i=i->parent) {
+	if (i->is_group() && prev_parent && 
+	    (!strcmp(i->type_name(), "Fl_Tabs") || 
+	     !strcmp(i->type_name(), "Fl_Wizard"))) {
+	     ((Fl_Tabs*)((Fl_Widget_Type*)i)->o)->value(prev_parent->o);
+	     return;
+	}
+	if (i->is_group() && s->is_widget()) 
+	    prev_parent = (Fl_Widget_Type*)i;
+    }
+}
+ 
 // do that for every window (when selected set changes):
 void redraw_overlays() {
   for (Fl_Type *o=Fl_Type::first; o; o=o->next)
@@ -1243,8 +1296,6 @@ int Fl_Window_Type::handle(int event) {
       ((Fl_Window*)o)->hide();
       return 1;
 
-    case 0xFE20: // backtab
-      backtab = 1;
     case FL_Tab: {
       if (Fl::event_state(FL_SHIFT)) backtab = 1;
       // find current child:
@@ -1451,18 +1502,32 @@ void Fl_Widget_Class_Type::write_code1() {
     write_h("public:\n");
     write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", name());
     write_h("  %s(int W, int H, const char *L = 0);\n", name());
+    write_h("  %s();\n", name());
 
+    // a constructor with all four dimensions plus label
     write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), name());
     write_c("  : %s(X, Y, W, H, L) {\n", c);
     write_c("  _%s();\n", name());
     write_c("}\n\n");
 
+    // a constructor with just the size and label. The window manager will position the window
     write_c("%s::%s(int W, int H, const char *L)\n", name(), name());
     write_c("  : %s(0, 0, W, H, L) {\n", c);
     write_c("  clear_flag(16);\n");
     write_c("  _%s();\n", name());
     write_c("}\n\n");
-
+    
+    // a constructor that takes size and label from the Fluid database
+    write_c("%s::%s()\n", name(), name());
+    write_c("  : %s(0, 0, %d, %d, ", c, o->w(), o->h());
+    const char *cstr = label();
+    if (cstr) write_cstring(cstr);
+    else write_c("0");
+    write_c(") {\n");
+    write_c("  clear_flag(16);\n");
+    write_c("  _%s();\n", name());
+    write_c("}\n\n");
+    
     write_c("void %s::_%s() {\n", name(), name());
 //    write_c("  %s *w = this;\n", name());
   } else {
@@ -1529,5 +1594,5 @@ void Fl_Window_Type::copy_properties() {
 
 
 //
-// End of "$Id: Fl_Window_Type.cxx 6098 2008-04-12 18:04:09Z matt $".
+// End of "$Id: Fl_Window_Type.cxx 8089 2010-12-20 22:19:24Z matt $".
 //
