@@ -1,5 +1,5 @@
 //
-// "$Id: file.cxx 5808 2007-05-10 12:06:31Z matt $"
+// "$Id: file.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Fluid file routines for the Fast Light Tool Kit (FLTK).
 //
@@ -8,7 +8,7 @@
 // They are somewhat similar to tcl, using matching { and }
 // to quote strings.
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -43,7 +43,7 @@ static FILE *fout;
 
 int open_write(const char *s) {
   if (!s) {fout = stdout; return 1;}
-  FILE *f = fopen(s,"w");
+  FILE *f = fl_fopen(s,"w");
   if (!f) return 0;
   fout = f;
   return 1;
@@ -135,7 +135,7 @@ static const char *fname;
 int open_read(const char *s) {
   lineno = 1;
   if (!s) {fin = stdin; fname = "stdin"; return 1;}
-  FILE *f = fopen(s,"r");
+  FILE *f = fl_fopen(s,"r");
   if (!f) return 0;
   fin = f;
   fname = s;
@@ -159,7 +159,7 @@ void read_error(const char *format, ...) {
   if (!fin) {
     char buffer[1024];
     vsnprintf(buffer, sizeof(buffer), format, args);
-    fl_message(buffer);
+    fl_message("%s", buffer);
   } else {
     fprintf(stderr, "%s:%d: ", fname, lineno);
     vfprintf(stderr, format, args);
@@ -389,11 +389,11 @@ static void read_children(Fl_Type *p, int paste) {
       c = read_word();
       read_version = strtod(c,0);
       if (read_version<=0 || read_version>FL_VERSION)
-	read_error("unknown version '%s'",c);
+        read_error("unknown version '%s'",c);
       continue;
     }
 
-    // back compatability with Vincent Penne's original class code:
+    // back compatibility with Vincent Penne's original class code:
     if (!p && !strcmp(c,"define_in_struct")) {
       Fl_Type *t = Fl_Type_make("class");
       t->name(read_word());
@@ -646,5 +646,5 @@ void read_fdesign() {
 }
 
 //
-// End of "$Id: file.cxx 5808 2007-05-10 12:06:31Z matt $".
+// End of "$Id: file.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
