@@ -1,9 +1,9 @@
 //
-// "$Id: fluid.cxx 6783 2009-04-27 15:50:29Z AlbrechtS $"
+// "$Id: fluid.cxx 8801 2011-06-10 13:37:07Z manolo $"
 //
 // FLUID main entry for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2007 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -34,9 +34,11 @@
 #include <FL/Fl_Hold_Browser.H>
 #include <FL/Fl_Menu_Bar.H>
 #include <FL/Fl_Input.H>
+#include <FL/Fl_Plugin.H>
 #include <FL/fl_ask.H>
 #include <FL/fl_draw.H>
 #include <FL/Fl_File_Chooser.H>
+#include <FL/Fl_PNG_Image.H>
 #include <FL/fl_message.H>
 #include <FL/filename.H>
 #include <stdio.h>
@@ -57,6 +59,7 @@
 #  include <direct.h>
 #  include <windows.h>
 #  include <io.h>
+#  include <fcntl.h>
 #  include <commdlg.h>
 #  include <FL/x.H>
 #  ifndef __WATCOMC__
@@ -97,10 +100,12 @@ int gridx = 5;
 int gridy = 5;
 int snap = 1;
 int show_guides = 1;
+int show_comments = 1;
+int show_coredevmenus = 1;
 
 // File history info...
-char	absolute_history[10][1024];
-char	relative_history[10][1024];
+char	absolute_history[10][FL_PATH_MAX];
+char	relative_history[10][FL_PATH_MAX];
 
 void	load_history();
 void	update_history(const char *);
@@ -108,6 +113,11 @@ void	update_history(const char *);
 // Shell command support...
 void	show_shell_window();
 
+Fl_Menu_Item *save_item = 0L;
+Fl_Menu_Item *history_item = 0L;
+Fl_Menu_Item *widgetbin_item = 0L;
+Fl_Menu_Item *sourceview_item = 0L;
+
 ////////////////////////////////////////////////////////////////
 
 static const char *filename;
@@ -122,11 +132,11 @@ void goto_source_dir() {
   if (!filename || !*filename) return;
   const char *p = fl_filename_name(filename);
   if (p <= filename) return; // it is in the current directory
-  char buffer[1024];
+  char buffer[FL_PATH_MAX];
   strlcpy(buffer, filename, sizeof(buffer));
   int n = p-filename; if (n>1) n--; buffer[n] = 0;
   if (!pwd) {
-    pwd = getcwd(0,1024);
+    pwd = getcwd(0,FL_PATH_MAX);
     if (!pwd) {fprintf(stderr,"getwd : %s\n",strerror(errno)); return;}
   }
   if (chdir(buffer)<0) {fprintf(stderr, "Can't chdir to %s : %s\n",
@@ -140,7 +150,7 @@ void leave_source_dir() {
 			     pwd, strerror(errno));}
   in_source_dir = 0;
 }
-  
+
 char position_window(Fl_Window *w, const char *prefsName, int Visible, int X, int Y, int W=0, int H=0 ) {
   Fl_Preferences pos(fluid_prefs, prefsName);
   if (prevpos_button->value()) {
@@ -171,7 +181,7 @@ Fl_Window *main_window;
 Fl_Menu_Bar *main_menubar;
 
 static char* cutfname(int which = 0) {
-  static char name[2][1024];
+  static char name[2][FL_PATH_MAX];
   static char beenhere = 0;
 
   if (!beenhere) {
@@ -252,14 +262,14 @@ void save_template_cb(Fl_Widget *, void *) {
   if (!c || !*c) return;
 
   // Convert template name to filename_with_underscores
-  char safename[1024], *safeptr;
+  char safename[FL_PATH_MAX], *safeptr;
   strlcpy(safename, c, sizeof(safename));
   for (safeptr = safename; *safeptr; safeptr ++) {
     if (isspace(*safeptr)) *safeptr = '_';
   }
 
   // Find the templates directory...
-  char filename[1024];
+  char filename[FL_PATH_MAX];
   fluid_prefs.getUserdataPath(filename, sizeof(filename));
 
   strlcat(filename, "templates", sizeof(filename));
@@ -315,7 +325,7 @@ void save_template_cb(Fl_Widget *, void *) {
 
   FILE *fp;
 
-  if ((fp = fopen(filename, "wb")) == NULL) {
+  if ((fp = fl_fopen(filename, "wb")) == NULL) {
     delete[] pixels;
     fl_alert("Error writing %s: %s", filename, strerror(errno));
     return;
@@ -343,7 +353,7 @@ void save_template_cb(Fl_Widget *, void *) {
 
 #  if 0 // The original PPM output code...
   strcpy(ext, ".ppm");
-  fp = fopen(filename, "wb");
+  fp = fl_fopen(filename, "wb");
   fprintf(fp, "P6\n%d %d 255\n", w, h);
   fwrite(pixels, w * h, 3, fp);
   fclose(fp);
@@ -584,7 +594,7 @@ void new_cb(Fl_Widget *, void *v) {
       char line[1024], *ptr, *next;
       FILE *infile, *outfile;
 
-      if ((infile = fopen(tname, "r")) == NULL) {
+      if ((infile = fl_fopen(tname, "r")) == NULL) {
 	fl_alert("Error reading template file \"%s\":\n%s", tname,
         	 strerror(errno));
 	set_modflag(0);
@@ -592,7 +602,7 @@ void new_cb(Fl_Widget *, void *v) {
 	return;
       }
 
-      if ((outfile = fopen(cutfname(1), "w")) == NULL) {
+      if ((outfile = fl_fopen(cutfname(1), "w")) == NULL) {
 	fl_alert("Error writing buffer file \"%s\":\n%s", cutfname(1),
         	 strerror(errno));
 	fclose(infile);
@@ -630,6 +640,7 @@ void new_cb(Fl_Widget *, void *v) {
   undo_clear();
 }
 
+int exit_early = 0;
 int compile_only = 0;
 int compile_strings = 0;
 int header_file_set = 0;
@@ -641,15 +652,15 @@ const char* i18n_include = "";
 const char* i18n_function = "";
 const char* i18n_file = "";
 const char* i18n_set = "";
-char i18n_program[1024] = "";
+char i18n_program[FL_PATH_MAX] = "";
 
 void write_cb(Fl_Widget *, void *) {
   if (!filename) {
     save_cb(0,0);
     if (!filename) return;
   }
-  char cname[1024];
-  char hname[1024];
+  char cname[FL_PATH_MAX];
+  char hname[FL_PATH_MAX];
   strlcpy(i18n_program, fl_filename_name(filename), sizeof(i18n_program));
   fl_filename_setext(i18n_program, sizeof(i18n_program), "");
   if (*code_file_name == '.' && strchr(code_file_name, '/') == NULL) {
@@ -686,7 +697,7 @@ void write_strings_cb(Fl_Widget *, void *) {
     save_cb(0,0);
     if (!filename) return;
   }
-  char sname[1024];
+  char sname[FL_PATH_MAX];
   strlcpy(sname, fl_filename_name(filename), sizeof(sname));
   fl_filename_setext(sname, sizeof(sname), exts[i18n_type]);
   if (!compile_only) goto_source_dir();
@@ -825,6 +836,7 @@ static void sort_cb(Fl_Widget *,void *) {
 void show_project_cb(Fl_Widget *, void *);
 void show_grid_cb(Fl_Widget *, void *);
 void show_settings_cb(Fl_Widget *, void *);
+void show_global_settings_cb(Fl_Widget *, void *);
 
 void align_widget_cb(Fl_Widget *, long);
 void widget_size_cb(Fl_Widget *, long);
@@ -836,14 +848,14 @@ void about_cb(Fl_Widget *, void *) {
 
 void show_help(const char *name) {
   const char	*docdir;
-  char		helpname[1024];
-  
+  char		helpname[FL_PATH_MAX];
+
   if (!help_dialog) help_dialog = new Fl_Help_Dialog();
 
   if ((docdir = getenv("FLTK_DOCDIR")) == NULL) {
 #ifdef __EMX__
     // Doesn't make sense to have a hardcoded fallback
-    static char fltk_docdir[1024];
+    static char fltk_docdir[FL_PATH_MAX];
 
     strlcpy(fltk_docdir, __XOS2RedirRoot("/XFree86/lib/X11/fltk/doc"),
             sizeof(fltk_docdir));
@@ -853,9 +865,58 @@ void show_help(const char *name) {
     docdir = FLTK_DOCDIR;
 #endif // __EMX__
   }
-  snprintf(helpname, sizeof(helpname), "%s/%s", docdir, name);  
+  snprintf(helpname, sizeof(helpname), "%s/%s", docdir, name);
 
-  help_dialog->load(helpname);
+  // make sure that we can read the file
+  FILE *f = fopen(helpname, "rb");
+  if (f) {
+    fclose(f);
+    help_dialog->load(helpname);
+  } else {
+    // if we can not read the file, we display the canned version instead
+    // or ask the native browser to open the page on www.fltk.org
+    if (strcmp(name, "fluid.html")==0) {
+      if (!Fl_Shared_Image::find("embedded:/fluid-org.png"))
+        new Fl_PNG_Image("embedded:/fluid-org.png", fluid_org_png, sizeof(fluid_org_png));
+      help_dialog->value
+      (
+       "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
+       "<html><head><title>FLTK: Programming with FLUID</title></head><body>\n"
+       "<h2>What is FLUID?</h2>\n"
+       "The Fast Light User Interface Designer, or FLUID, is a graphical editor "
+       "that is used to produce FLTK source code. FLUID edits and saves its state "
+       "in <code>.fl</code> files. These files are text, and you can (with care) "
+       "edit them in a text editor, perhaps to get some special effects.<p>\n"
+       "FLUID can \"compile\" the <code>.fl</code> file into a <code>.cxx</code> "
+       "and a <code>.h</code> file. The <code>.cxx</code> file defines all the "
+       "objects from the <code>.fl</code> file and the <code>.h</code> file "
+       "declares all the global ones. FLUID also supports localization "
+       "(Internationalization) of label strings using message files and the GNU "
+       "gettext or POSIX catgets interfaces.<p>\n"
+       "A simple program can be made by putting all your code (including a <code>"
+       "main()</code> function) into the <code>.fl</code> file and thus making the "
+       "<code>.cxx</code> file a single source file to compile. Most programs are "
+       "more complex than this, so you write other <code>.cxx</code> files that "
+       "call the FLUID functions. These <code>.cxx</code> files must <code>"
+       "#include</code> the <code>.h</code> file or they can <code>#include</code> "
+       "the <code>.cxx</code> file so it still appears to be a single source file.<p>"
+       "<img src=\"embedded:/fluid-org.png\"></p>"
+       "<p>More information is available online at <a href="
+       "\"http://www.fltk.org/doc-1.3/fluid.html\">http://www.fltk.org/</a>"
+       "</body></html>"
+       );
+    } else if (strcmp(name, "license.html")==0) {
+      fl_open_uri("http://www.fltk.org/doc-1.3/license.html");
+      return;
+    } else if (strcmp(name, "index.html")==0) {
+      fl_open_uri("http://www.fltk.org/doc-1.3/index.html");
+      return;
+    } else {
+      snprintf(helpname, sizeof(helpname), "http://www.fltk.org/%s", name);
+      fl_open_uri(helpname);
+      return;
+    }
+  }
   help_dialog->show();
 }
 
@@ -989,7 +1050,7 @@ void print_menu_cb(Fl_Widget *, void *) {
   char date[1024];
 
   strftime(date, sizeof(date), "%c", curdate);
-    
+
   // Print each of the windows...
   for (winpage = 0; winpage < num_windows; winpage ++) {
     // Draw header...
@@ -1140,7 +1201,7 @@ void print_menu_cb(Fl_Widget *, void *) {
 // Quote a string for PostScript printing
 static const char *ps_string(const char *s) {
   char *bufptr;
-  static char buffer[2048];
+  static char buffer[FL_PATH_MAX];
 
 
   if (!s) {
@@ -1199,7 +1260,7 @@ void print_cb(Fl_Return_Button *, void *) {
     if (t->is_window()) {
       winpage ++;
       windows[num_windows] = (Fl_Window_Type *)t;
-
+      if (!((Fl_Window*)(windows[num_windows]->o))->shown()) continue;
       if (print_all->value()) num_windows ++;
       else if (print_pages->value() && winpage >= first &&
                winpage <= last) num_windows ++;
@@ -1240,7 +1301,7 @@ void print_cb(Fl_Return_Button *, void *) {
 		    "Replace", NULL, outname) == 0) outname = NULL;
     }
 
-    if (outname) outfile = fopen(outname, "w");
+    if (outname) outfile = fl_fopen(outname, "w");
     else outfile = NULL;
   }
 
@@ -1285,7 +1346,7 @@ void print_cb(Fl_Return_Button *, void *) {
     char date[1024];
 
     strftime(date, sizeof(date), "%c", curdate);
-    
+
     // Write the prolog...
     fprintf(outfile,
             "%%!PS-Adobe-3.0\n"
@@ -1401,7 +1462,7 @@ void print_cb(Fl_Return_Button *, void *) {
         // Draw a simulated window border...
         fprintf(outfile,
 	        "0.75 setgray\n"			// Gray background
-	        "newpath %.2f %.2f %.2f 180 90 arcn\n"	// Top left 
+	        "newpath %.2f %.2f %.2f 180 90 arcn\n"	// Top left
 		"%.2f %.2f %.2f 90 0 arcn\n"		// Top right
 		"%.2f %.2f %.2f 0 -90 arcn\n"		// Bottom right
 		"%.2f %.2f %.2f -90 -180 arcn\n"	// Bottom left
@@ -1580,7 +1641,6 @@ Fl_Menu_Item Main_Menu[] = {
   {"&New...", FL_COMMAND+'n', new_cb, 0},
   {"&Open...", FL_COMMAND+'o', open_cb, 0},
   {"&Insert...", FL_COMMAND+'i', open_cb, (void*)1, FL_MENU_DIVIDER},
-#define SAVE_ITEM 4
   {"&Save", FL_COMMAND+'s', save_cb, 0},
   {"Save &As...", FL_COMMAND+FL_SHIFT+'s', save_cb, (void*)1},
   {"Sa&ve A Copy...", 0, save_cb, (void*)2},
@@ -1589,7 +1649,6 @@ Fl_Menu_Item Main_Menu[] = {
   {"&Print...", FL_COMMAND+'p', print_menu_cb},
   {"Write &Code...", FL_COMMAND+FL_SHIFT+'c', write_cb, 0},
   {"&Write Strings...", FL_COMMAND+FL_SHIFT+'w', write_strings_cb, 0, FL_MENU_DIVIDER},
-#define HISTORY_ITEM 12
   {relative_history[0], FL_COMMAND+'0', open_history_cb, absolute_history[0]},
   {relative_history[1], FL_COMMAND+'1', open_history_cb, absolute_history[1]},
   {relative_history[2], FL_COMMAND+'2', open_history_cb, absolute_history[2]},
@@ -1619,12 +1678,11 @@ Fl_Menu_Item Main_Menu[] = {
   {"&Group", FL_F+7, group_cb},
   {"Ung&roup", FL_F+8, ungroup_cb,0, FL_MENU_DIVIDER},
   {"Hide O&verlays",FL_COMMAND+FL_SHIFT+'o',toggle_overlays},
-#define WIDGETBIN_ITEM 41
   {"Show Widget &Bin...",FL_ALT+'b',toggle_widgetbin_cb},
-#define SOURCEVIEW_ITEM 42
   {"Show Source Code...",FL_ALT+FL_SHIFT+'s', (Fl_Callback*)toggle_sourceview_cb, 0, FL_MENU_DIVIDER},
   {"Pro&ject Settings...",FL_ALT+'p',show_project_cb},
-  {"GU&I Settings...",FL_ALT+FL_SHIFT+'p',show_settings_cb},
+  {"GU&I Settings...",FL_ALT+FL_SHIFT+'p',show_settings_cb,0,FL_MENU_DIVIDER},
+  {"Global &FLTK Settings...",FL_ALT+FL_SHIFT+'g',show_global_settings_cb},
   {0},
 {"&New", 0, 0, (void *)New_Menu, FL_SUBMENU_POINTER},
 {"&Layout",0,0,0,FL_SUBMENU},
@@ -1664,9 +1722,9 @@ Fl_Menu_Item Main_Menu[] = {
   {"Execute &Again...",FL_ALT+'g',(Fl_Callback *)do_shell_command},
   {0},
 {"&Help",0,0,0,FL_SUBMENU},
+  {"&Rapid development with FLUID...",0,help_cb},
+  {"&FLTK Programmers Manual...",0,manual_cb, 0, FL_MENU_DIVIDER},
   {"&About FLUID...",0,about_cb},
-  {"&On FLUID...",0,help_cb},
-  {"&Manual...",0,manual_cb},
   {0},
 {0}};
 
@@ -1703,16 +1761,15 @@ void scheme_cb(Fl_Choice *, void *) {
 void toggle_widgetbin_cb(Fl_Widget *, void *) {
   if (!widgetbin_panel) {
     make_widgetbin();
-    widgetbin_panel->callback(toggle_widgetbin_cb);
     if (!position_window(widgetbin_panel,"widgetbin_pos", 1, 320, 30)) return;
   }
 
   if (widgetbin_panel->visible()) {
     widgetbin_panel->hide();
-    Main_Menu[WIDGETBIN_ITEM].label("Show Widget &Bin...");
+    widgetbin_item->label("Show Widget &Bin...");
   } else {
     widgetbin_panel->show();
-    Main_Menu[WIDGETBIN_ITEM].label("Hide Widget &Bin");
+    widgetbin_item->label("Hide Widget &Bin");
   }
 }
 
@@ -1736,10 +1793,10 @@ void toggle_sourceview_cb(Fl_Double_Window *, void *) {
 
   if (sourceview_panel->visible()) {
     sourceview_panel->hide();
-    Main_Menu[SOURCEVIEW_ITEM].label("Show Source Code...");
+    sourceview_item->label("Show Source Code...");
   } else {
     sourceview_panel->show();
-    Main_Menu[SOURCEVIEW_ITEM].label("Hide Source Code...");
+    sourceview_item->label("Hide Source Code...");
     update_sourceview_cb(0,0);
   }
 }
@@ -1749,17 +1806,16 @@ void toggle_sourceview_b_cb(Fl_Button*, void *) {
 }
 
 void make_main_window() {
-  fluid_prefs.get("snap", snap, 1);
-  fluid_prefs.get("gridx", gridx, 5);
-  fluid_prefs.get("gridy", gridy, 5);
-  fluid_prefs.get("show_guides", show_guides, 0);
-  fluid_prefs.get("widget_size", Fl_Widget_Type::default_size, 14);
-
-  load_history();
-
-  make_layout_window();
-  make_settings_window();
-  make_shell_window();
+  if (!compile_only) {
+    fluid_prefs.get("snap", snap, 1);
+    fluid_prefs.get("gridx", gridx, 5);
+    fluid_prefs.get("gridy", gridy, 5);
+    fluid_prefs.get("show_guides", show_guides, 0);
+    fluid_prefs.get("widget_size", Fl_Widget_Type::default_size, 14);
+    fluid_prefs.get("show_comments", show_comments, 1);
+    make_layout_window();
+    make_shell_window();
+  }
 
   if (!main_window) {
     Fl_Widget *o;
@@ -1771,10 +1827,21 @@ void make_main_window() {
     main_window->resizable(o);
     main_menubar = new Fl_Menu_Bar(0,0,BROWSERWIDTH,MENUHEIGHT);
     main_menubar->menu(Main_Menu);
+    // quick access to all dynamic menu items
+    save_item = (Fl_Menu_Item*)main_menubar->find_item(save_cb);
+    history_item = (Fl_Menu_Item*)main_menubar->find_item(open_history_cb);
+    widgetbin_item = (Fl_Menu_Item*)main_menubar->find_item(toggle_widgetbin_cb);
+    sourceview_item = (Fl_Menu_Item*)main_menubar->find_item((Fl_Callback*)toggle_sourceview_cb);
     main_menubar->global();
     fill_in_New_Menu();
     main_window->end();
   }
+
+  if (!compile_only) {
+    load_history();
+    make_settings_window();
+    make_global_settings_window();
+  }
 }
 
 // Load file history from preferences...
@@ -1793,21 +1860,21 @@ void load_history() {
       fl_filename_relative(relative_history[i], sizeof(relative_history[i]),
                            absolute_history[i]);
 
-      if (i == 9) Main_Menu[i + HISTORY_ITEM].flags = FL_MENU_DIVIDER;
-      else Main_Menu[i + HISTORY_ITEM].flags = 0;
+      if (i == 9) history_item[i].flags = FL_MENU_DIVIDER;
+      else history_item[i].flags = 0;
     } else break;
   }
 
   for (; i < 10; i ++) {
-    if (i) Main_Menu[i + HISTORY_ITEM - 1].flags |= FL_MENU_DIVIDER;
-    Main_Menu[i + HISTORY_ITEM].hide();
+    if (i) history_item[i-1].flags |= FL_MENU_DIVIDER;
+    history_item[i].hide();
   }
 }
 
 // Update file history from preferences...
 void update_history(const char *flname) {
   int	i;		// Looping var
-  char	absolute[1024];
+  char	absolute[FL_PATH_MAX];
   int	max_files;
 
 
@@ -1843,36 +1910,177 @@ void update_history(const char *flname) {
   for (i = 0; i < max_files; i ++) {
     fluid_prefs.set( Fl_Preferences::Name("file%d", i), absolute_history[i]);
     if (absolute_history[i][0]) {
-      if (i == 9) Main_Menu[i + HISTORY_ITEM].flags = FL_MENU_DIVIDER;
-      else Main_Menu[i + HISTORY_ITEM].flags = 0;
+      if (i == 9) history_item[i].flags = FL_MENU_DIVIDER;
+      else history_item[i].flags = 0;
     } else break;
   }
 
   for (; i < 10; i ++) {
     fluid_prefs.set( Fl_Preferences::Name("file%d", i), "");
-    if (i) Main_Menu[i + HISTORY_ITEM - 1].flags |= FL_MENU_DIVIDER;
-    Main_Menu[i + HISTORY_ITEM].hide();
+    if (i) history_item[i-1].flags |= FL_MENU_DIVIDER;
+    history_item[i].hide();
   }
 }
 
-// Shell command support...
-#if (!defined(WIN32) || defined(__CYGWIN__)) && !defined(__MWERKS__)
-// Support the full piped shell command...
-static FILE *shell_pipe = 0;
+// ********** portable process class definition **********
 
+class Fl_Process {
+public:
+  // construction / destruction
+  Fl_Process() {_fpt= NULL;}
+  ~Fl_Process() {if (_fpt) close();}
+
+  // FIXME: popen needs the utf8 equivalen fl_popen
+  FILE * popen	(const char *cmd, const char *mode="r");
+  //not necessary here: FILE * fl_fopen	(const char *file, const char *mode="r");
+  int  close();
+
+  FILE * desc() const { return _fpt;} // non null if file is open
+  char * get_line(char * line, size_t s) const {return _fpt ? fgets(line, s, _fpt) : NULL;}
+
+#if defined(WIN32)  && !defined(__CYGWIN__)
+protected:
+  HANDLE pin[2], pout[2], perr[2];
+  char ptmode;
+  PROCESS_INFORMATION pi;
+  STARTUPINFO si;
+
+  static bool createPipe(HANDLE * h, BOOL bInheritHnd=TRUE);
+
+private:
+  FILE * freeHandles()  {
+    clean_close(pin[0]);    clean_close(pin[1]);
+    clean_close(pout[0]);   clean_close(pout[1]);
+    clean_close(perr[0]);   clean_close(perr[1]);
+    return NULL; // convenient for error management
+  }
+  static void clean_close(HANDLE& h);
+#endif
+
+protected:
+  FILE * _fpt;
+};
+
+#if defined(WIN32)  && !defined(__CYGWIN__)
+bool Fl_Process::createPipe(HANDLE * h, BOOL bInheritHnd) {
+  SECURITY_ATTRIBUTES sa;
+  sa.nLength = sizeof(sa);
+  sa.lpSecurityDescriptor = NULL;
+  sa.bInheritHandle = bInheritHnd;
+  return CreatePipe (&h[0],&h[1],&sa,0) ? true : false;
+}
+#endif
+// portable open process:
+FILE * Fl_Process::popen(const char *cmd, const char *mode) {
+#if defined(WIN32)  && !defined(__CYGWIN__)
+  // PRECONDITIONS
+  if (!mode || !*mode || (*mode!='r' && *mode!='w') ) return NULL;
+  if (_fpt) close(); // close first before reuse
+
+  ptmode = *mode;
+  pin[0] = pin[1] = pout[0] = pout[1] = perr[0] = perr[1] = INVALID_HANDLE_VALUE;
+  // stderr to stdout wanted ?
+  int fusion = (strstr(cmd,"2>&1") !=NULL);
+
+  // Create windows pipes
+  if (!createPipe(pin) || !createPipe(pout) || (!fusion && !createPipe(perr) ) )
+	return freeHandles(); // error
+
+  // Initialize Startup Info
+  ZeroMemory(&si, sizeof(STARTUPINFO));
+  si.cb           = sizeof(STARTUPINFO);
+  si.dwFlags    = STARTF_USESTDHANDLES;
+  si.hStdInput    = pin[0];
+  si.hStdOutput   = pout[1];
+  si.hStdError  = fusion ? pout[1] : perr [1];
+
+  if ( CreateProcess(NULL, (LPTSTR) cmd,NULL,NULL,TRUE,
+		     DETACHED_PROCESS,NULL,NULL, &si, &pi)) {
+    // don't need theses handles inherited by child process:
+    clean_close(pin[0]); clean_close(pout[1]); clean_close(perr[1]);
+    HANDLE & h = *mode == 'r' ? pout[0] : pin[1];
+    _fpt = _fdopen(_open_osfhandle((fl_intptr_t) h,_O_BINARY),mode);
+    h= INVALID_HANDLE_VALUE;  // reset the handle pointer that is shared
+    // with _fpt so we don't free it twice
+  }
+
+  if (!_fpt)  freeHandles();
+  return _fpt;
+#else
+  _fpt=::popen(cmd,mode);
+  return _fpt;
+#endif
+}
+
+int Fl_Process::close() {
+#if defined(WIN32)  && !defined(__CYGWIN__)
+  if (_fpt) {
+    fclose(_fpt);
+    clean_close(perr[0]);
+    clean_close(pin[1]);
+    clean_close(pout[0]);
+    _fpt = NULL;
+    return 0;
+  }
+  return -1;
+#else
+  int ret = ::pclose(_fpt);
+  _fpt=NULL;
+  return ret;
+#endif
+}
+
+#if defined(WIN32)  && !defined(__CYGWIN__)
+void Fl_Process::clean_close(HANDLE& h) {
+  if (h!= INVALID_HANDLE_VALUE) CloseHandle(h);
+  h = INVALID_HANDLE_VALUE;
+}
+#endif
+// ********** Fl_Process class end **********
+
+static Fl_Process s_proc;
+
+// Shell command support...
+
+static bool prepare_shell_command(const char * &command)  { // common pre-shell command code all platforms
+  shell_window->hide();
+  if (s_proc.desc()) {
+    fl_alert("Previous shell command still running!");
+    return false;
+  }
+  if ((command = shell_command_input->value()) == NULL || !*command) {
+    fl_alert("No shell command entered!");
+    return false;
+  }
+  if (shell_savefl_button->value()) {
+    save_cb(0, 0);
+  }
+  if (shell_writecode_button->value()) {
+    compile_only = 1;
+    write_cb(0, 0);
+    compile_only = 0;
+  }
+  if (shell_writemsgs_button->value()) {
+    compile_only = 1;
+    write_strings_cb(0, 0);
+    compile_only = 0;
+  }
+  return true;
+}
+
+#if !defined(__MWERKS__)
+// Support the full piped shell command...
 void
 shell_pipe_cb(int, void*) {
-  char	line[1024];		// Line from command output...
+  char	line[1024]="";		// Line from command output...
 
-  if (fgets(line, sizeof(line), shell_pipe) != NULL) {
+  if (s_proc.get_line(line, sizeof(line)) != NULL) {
     // Add the line to the output list...
     shell_run_buffer->append(line);
   } else {
     // End of file; tell the parent...
-    Fl::remove_fd(fileno(shell_pipe));
-
-    pclose(shell_pipe);
-    shell_pipe = NULL;
+    Fl::remove_fd(fileno(s_proc.desc()));
+    s_proc.close();
     shell_run_buffer->append("... END SHELL COMMAND ...\n");
   }
 
@@ -1882,36 +2090,9 @@ shell_pipe_cb(int, void*) {
 
 void
 do_shell_command(Fl_Return_Button*, void*) {
-  const char	*command;	// Command to run
+  const char	*command=NULL;	// Command to run
 
-
-  shell_window->hide();
-
-  if (shell_pipe) {
-    fl_alert("Previous shell command still running!");
-    return;
-  }
-
-  if ((command = shell_command_input->value()) == NULL || !*command) {
-    fl_alert("No shell command entered!");
-    return;
-  }
-
-  if (shell_savefl_button->value()) {
-    save_cb(0, 0);
-  }
-
-  if (shell_writecode_button->value()) {
-    compile_only = 1;
-    write_cb(0, 0);
-    compile_only = 0;
-  }
-
-  if (shell_writemsgs_button->value()) {
-    compile_only = 1;
-    write_strings_cb(0, 0);
-    compile_only = 0;
-  }
+  if (!prepare_shell_command(command)) return;
 
   // Show the output window and clear things...
   shell_run_buffer->text("");
@@ -1919,7 +2100,7 @@ do_shell_command(Fl_Return_Button*, void*) {
   shell_run_buffer->append("\n");
   shell_run_window->label("Shell Command Running...");
 
-  if ((shell_pipe = popen((char *)command, "r")) == NULL) {
+  if (s_proc.popen((char *)command) == NULL) {
     fl_alert("Unable to run shell command: %s", strerror(errno));
     return;
   }
@@ -1928,9 +2109,9 @@ do_shell_command(Fl_Return_Button*, void*) {
   shell_run_window->hotspot(shell_run_display);
   shell_run_window->show();
 
-  Fl::add_fd(fileno(shell_pipe), shell_pipe_cb);
+  Fl::add_fd(fileno(s_proc.desc()), shell_pipe_cb);
 
-  while (shell_pipe) Fl::wait();
+  while (s_proc.desc()) Fl::wait();
 
   shell_run_button->activate();
   shell_run_window->label("Shell Command Complete");
@@ -1945,29 +2126,7 @@ do_shell_command(Fl_Return_Button*, void*) {
   const char	*command;	// Command to run
   int		status;		// Status from command...
 
-
-  shell_window->hide();
-
-  if ((command = shell_command_input->value()) == NULL || !*command) {
-    fl_alert("No shell command entered!");
-    return;
-  }
-
-  if (shell_savefl_button->value()) {
-    save_cb(0, 0);
-  }
-
-  if (shell_writecode_button->value()) {
-    compile_only = 1;
-    write_cb(0, 0);
-    compile_only = 0;
-  }
-
-  if (shell_writemsgs_button->value()) {
-    compile_only = 1;
-    write_strings_cb(0, 0);
-    compile_only = 0;
-  }
+  if (!prepare_shell_command(command)) return;
 
   if ((status = system(command)) != 0) {
     fl_alert("Shell command returned status %d!", status);
@@ -1975,8 +2134,7 @@ do_shell_command(Fl_Return_Button*, void*) {
     fl_message("Shell command completed successfully!");
   }
 }
-#endif // (!WIN32 || __CYGWIN__) && !__MWERKS__
-
+#endif // !__MWERKS__
 
 void
 show_shell_window() {
@@ -1994,7 +2152,7 @@ void set_filename(const char *c) {
 }
 
 //
-// The Source View system offers an immediate preview of the code 
+// The Source View system offers an immediate preview of the code
 // files that will be generated by FLUID. It also marks the code
 // generated for the last selected item in the header and the source
 // file.
@@ -2008,15 +2166,15 @@ void set_filename(const char *c) {
 //
 void update_sourceview_position()
 {
-  if (!sourceview_panel || !sourceview_panel->visible()) 
+  if (!sourceview_panel || !sourceview_panel->visible())
     return;
-  if (sv_autoposition->value()==0) 
+  if (sv_autoposition->value()==0)
     return;
   if (sourceview_panel && sourceview_panel->visible() && Fl_Type::current) {
     int pos0, pos1;
     if (sv_source->visible_r()) {
-      pos0 = Fl_Type::current->code_line;
-      pos1 = Fl_Type::current->code_line_end;
+      pos0 = Fl_Type::current->code_position;
+      pos1 = Fl_Type::current->code_position_end;
       if (pos0>=0) {
         if (pos1<pos0)
           pos1 = pos0;
@@ -2026,8 +2184,8 @@ void update_sourceview_position()
       }
     }
     if (sv_header->visible_r()) {
-      pos0 = Fl_Type::current->header_line;
-      pos1 = Fl_Type::current->header_line_end;
+      pos0 = Fl_Type::current->header_position;
+      pos1 = Fl_Type::current->header_position_end;
       if (pos0>=0) {
         if (pos1<pos0)
           pos1 = pos0;
@@ -2039,7 +2197,7 @@ void update_sourceview_position()
   }
 }
 
-void update_sourceview_position_cb(Fl_Tabs*, void*) 
+void update_sourceview_position_cb(Fl_Tabs*, void*)
 {
   update_sourceview_position();
 }
@@ -2051,9 +2209,9 @@ static char *sv_header_filename = 0;
 // Generate a header and source file in a temporary directory and
 // load those into the Code Viewer widgets.
 //
-void update_sourceview_cb(Fl_Button*, void*) 
+void update_sourceview_cb(Fl_Button*, void*)
 {
-  if (!sourceview_panel || !sourceview_panel->visible()) 
+  if (!sourceview_panel || !sourceview_panel->visible())
     return;
   // generate space for the source and header file filenames
   if (!sv_source_filename) {
@@ -2077,7 +2235,7 @@ void update_sourceview_cb(Fl_Button*, void*)
   // generate the code and load the files
   write_sourceview = 1;
   // generate files
-  if (write_code(sv_source_filename, sv_header_filename)) 
+  if (write_code(sv_source_filename, sv_header_filename))
   {
     // load file into source editor
     int pos = sv_source->top_line();
@@ -2096,7 +2254,7 @@ void update_sourceview_cb(Fl_Button*, void*)
   header_file_name = header_file_name_bak;
 }
 
-void update_sourceview_timer(void*) 
+void update_sourceview_timer(void*)
 {
   update_sourceview_cb(0,0);
 }
@@ -2104,14 +2262,14 @@ void update_sourceview_timer(void*)
 // Set the "modified" flag and update the title of the main window...
 void set_modflag(int mf) {
   const char	*basename;
-  static char	title[1024];
+  static char	title[FL_PATH_MAX];
 
   modflag = mf;
 
   if (main_window) {
     if (!filename) basename = "Untitled.fl";
     else if ((basename = strrchr(filename, '/')) != NULL) basename ++;
-#if defined(WIN32) || defined(__EMX__) 
+#if defined(WIN32) || defined(__EMX__)
     else if ((basename = strrchr(filename, '\\')) != NULL) basename ++;
 #endif // WIN32 || __EMX__
     else basename = filename;
@@ -2132,8 +2290,8 @@ void set_modflag(int mf) {
   }
 
   // Enable/disable the Save menu item...
-  if (modflag) Main_Menu[SAVE_ITEM].activate();
-  else Main_Menu[SAVE_ITEM].deactivate();
+  if (modflag) save_item->activate();
+  else save_item->deactivate();
 }
 
 ////////////////////////////////////////////////////////////////
@@ -2153,6 +2311,13 @@ static int arg(int argc, char** argv, int& i) {
     i += 2;
     return 2;
   }
+  Fl_Plugin_Manager pm("commandline");
+  int j, n = pm.plugins();
+  for (j=0; j<n; j++) {
+    Fl_Commandline_Plugin *pi = (Fl_Commandline_Plugin*)pm.plugin(j);
+    int r = pi->arg(argc, argv, i);
+    if (r) return r;
+  }
   return 0;
 }
 
@@ -2178,29 +2343,54 @@ static void sigint(SIGARG) {
 }
 #endif
 
+
 int main(int argc,char **argv) {
   int i = 1;
+  
   if (!Fl::args(argc,argv,i,arg) || i < argc-1) {
-    fprintf(stderr,"usage: %s <switches> name.fl\n"
-" -c : write .cxx and .h and exit\n"
-" -cs : write .cxx and .h and strings and exit\n"
-" -o <name> : .cxx output filename, or extension if <name> starts with '.'\n"
-" -h <name> : .h output filename, or extension if <name> starts with '.'\n"
-"%s\n", argv[0], Fl::help);
+    static const char *msg = 
+      "usage: %s <switches> name.fl\n"
+      " -c : write .cxx and .h and exit\n"
+      " -cs : write .cxx and .h and strings and exit\n"
+      " -o <name> : .cxx output filename, or extension if <name> starts with '.'\n"
+      " -h <name> : .h output filename, or extension if <name> starts with '.'\n";
+    int len = strlen(msg) + strlen(argv[0]) + strlen(Fl::help);
+    Fl_Plugin_Manager pm("commandline");
+    int i, n = pm.plugins();
+    for (i=0; i<n; i++) {
+      Fl_Commandline_Plugin *pi = (Fl_Commandline_Plugin*)pm.plugin(i);
+      if (pi) len += strlen(pi->help());
+    }
+    char *buf = (char*)malloc(len+1);
+    sprintf(buf, msg, argv[0]);
+    for (i=0; i<n; i++) {
+      Fl_Commandline_Plugin *pi = (Fl_Commandline_Plugin*)pm.plugin(i);
+      if (pi) strcat(buf, pi->help());
+    }
+    strcat(buf, Fl::help);
+#ifdef _MSC_VER
+    fl_message("%s\n", buf);
+#else
+    fprintf(stderr, "%s\n", buf);
+#endif
+    free(buf);
     return 1;
   }
+  if (exit_early)
+    exit(0);
+  
   const char *c = argv[i];
 
   fl_register_images();
 
   make_main_window();
 
-#ifdef __APPLE__
-  fl_open_callback(apple_open_cb);
-#endif // __APPLE__
 
   if (c) set_filename(c);
   if (!compile_only) {
+#ifdef __APPLE__
+    fl_open_callback(apple_open_cb);
+#endif // __APPLE__
     Fl::visual((Fl_Mode)(FL_DOUBLE|FL_INDEX));
     Fl_File_Icon::load_system_icons();
     main_window->callback(exit_cb);
@@ -2249,5 +2439,5 @@ int main(int argc,char **argv) {
 }
 
 //
-// End of "$Id: fluid.cxx 6783 2009-04-27 15:50:29Z AlbrechtS $".
+// End of "$Id: fluid.cxx 8801 2011-06-10 13:37:07Z manolo $".
 //
