@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_GIF_Image.cxx 5845 2007-05-20 00:01:06Z mike $"
+// "$Id: Fl_GIF_Image.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Fl_GIF_Image routines.
 //
-// Copyright 1997-2005 by Bill Spitzak and others.
+// Copyright 1997-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -36,6 +36,7 @@
 #include <FL/Fl_GIF_Image.H>
 #include <stdio.h>
 #include <stdlib.h>
+#include <FL/fl_utf8.h>
 #include "flstring.h"
 
 // Read a .gif file and convert it to a "xpm" format (actually my
@@ -78,11 +79,16 @@ typedef unsigned char uchar;
 #define NEXTBYTE (uchar)getc(GifFile)
 #define GETSHORT(var) var = NEXTBYTE; var += NEXTBYTE << 8
 
+/**  
+  The constructor loads the named GIF image.
+  <P>The inherited destructor free all memory and server resources that are used by
+  the image.
+*/
 Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
   FILE *GifFile;	// File to read
   char **new_data;	// Data array
 
-  if ((GifFile = fopen(infname, "rb")) == NULL) {
+  if ((GifFile = fl_fopen(infname, "rb")) == NULL) {
     Fl::error("Fl_GIF_Image: Unable to open %s!", infname);
     return;
   }
@@ -380,5 +386,5 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
 
 
 //
-// End of "$Id: Fl_GIF_Image.cxx 5845 2007-05-20 00:01:06Z mike $".
+// End of "$Id: Fl_GIF_Image.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
