@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_PNM_Image.cxx 6006 2007-12-20 16:32:56Z matt $"
+// "$Id: Fl_PNM_Image.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Fl_PNM_Image routines.
 //
-// Copyright 1997-2005 by Easy Software Products.
+// Copyright 1997-2010 by Easy Software Products.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,6 +25,7 @@
 //     http://www.fltk.org/str.php
 //
 // Contents:
+
 //
 //   Fl_PNM_Image::Fl_PNM_Image() - Load a PNM image...
 //
@@ -37,6 +38,7 @@
 #include <FL/Fl_PNM_Image.H>
 #include <stdio.h>
 #include <stdlib.h>
+#include <FL/fl_utf8.h>
 #include "flstring.h"
 
 
@@ -44,6 +46,11 @@
 // 'Fl_PNM_Image::Fl_PNM_Image()' - Load a PNM image...
 //
 
+/** 
+  The constructor loads the named PNM image. 
+  <P>The inherited destructor free all memory and server resources that are used by the image.
+*/
+
 Fl_PNM_Image::Fl_PNM_Image(const char *name)	// I - File to read
   : Fl_RGB_Image(0,0,0) {
   FILE		*fp;		// File pointer
@@ -58,7 +65,7 @@ Fl_PNM_Image::Fl_PNM_Image(const char *name)	// I - File to read
 		maxval;		// Maximum pixel value
 
 
-  if ((fp = fopen(name, "rb")) == NULL) return;
+  if ((fp = fl_fopen(name, "rb")) == NULL) return;
 
   //
   // Read the file header in the format:
@@ -159,7 +166,7 @@ Fl_PNM_Image::Fl_PNM_Image(const char *name)	// I - File to read
       case 5 :
       case 6 :
         if (maxval < 256) {
-          fread(ptr, w(), d(), fp);
+          if (fread(ptr, w(), d(), fp)) { /* ignored */ }
         } else {
           for (x = d() * w(); x > 0; x --) {
             val = (uchar)getc(fp);
@@ -186,5 +193,5 @@ Fl_PNM_Image::Fl_PNM_Image(const char *name)	// I - File to read
 
 
 //
-// End of "$Id: Fl_PNM_Image.cxx 6006 2007-12-20 16:32:56Z matt $".
+// End of "$Id: Fl_PNM_Image.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
