@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Tiled_Image.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: Fl_Tiled_Image.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Tiled image code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,17 +25,15 @@
 //     http://www.fltk.org/str.php
 //
 
+
 #include <FL/Fl.H>
 #include <FL/Fl_Tiled_Image.H>
 #include <FL/fl_draw.H>
 
-
-//
-// 'Fl_Tiled_Image::Fl_Tiled_Image()' - Constructor.
-//
-// Use a width and height of 0 to tile the whole window/widget.
-//
-
+/**
+  The constructors create a new tiled image containing the specified image.
+  Use a width and height of 0 to tile the whole window/widget.
+*/
 Fl_Tiled_Image::Fl_Tiled_Image(Fl_Image *i,	// I - Image to tile
                                int      W,	// I - Width of tiled area
 			       int      H) :	// I - Height of tiled area
@@ -46,13 +44,11 @@ Fl_Tiled_Image::Fl_Tiled_Image(Fl_Image *i,	// I - Image to tile
   if (W == 0) w(Fl::w());
   if (H == 0) h(Fl::h());
 }
-
-
-//
-// 'Fl_Tiled_Image::~Fl_Tiled_Image()' - Destructor.
-//
-
-Fl_Tiled_Image::~Fl_Tiled_Image() {
+/**
+  The destructor frees all memory and server resources that are used by
+  the tiled image.
+*/
+  Fl_Tiled_Image::~Fl_Tiled_Image() {
   if (alloc_image_) delete image_;
 }
 
@@ -115,7 +111,7 @@ Fl_Tiled_Image::draw(int X,	// I - Starting X position
   if (W == 0) W = Fl::w();
   if (H == 0) H = Fl::h();
 
-  fl_clip(X, Y, W, H);
+  fl_push_clip(X, Y, W, H);
 
   X += cx;
   Y += cy;
@@ -135,5 +131,5 @@ Fl_Tiled_Image::draw(int X,	// I - Starting X position
 
 
 //
-// End of "$Id: Fl_Tiled_Image.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: Fl_Tiled_Image.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
