@@ -1,9 +1,9 @@
 //
-// "$Id: fl_show_colormap.cxx 6926 2009-11-07 21:01:08Z matt $"
+// "$Id: fl_show_colormap.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Colormap color selection dialog for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,9 +25,6 @@
 //     http://www.fltk.org/str.php
 //
 
-// Select a color from the colormap.
-// Pretty much unchanged from Forms.
-
 #include <FL/Fl.H>
 #include <FL/Fl_Single_Window.H>
 #include <FL/fl_draw.H>
@@ -37,6 +34,10 @@
 #define BOXSIZE 14
 #define BORDER 4
 
+/** 
+ This widget creates a modal window for selecting a color from the colormap.
+ Pretty much unchanged from Forms.
+*/
 class ColorMenu : public Fl_Window {
   Fl_Color initial;
   Fl_Color which, previous;
@@ -53,8 +54,7 @@ ColorMenu::ColorMenu(Fl_Color oldcol) :
   Fl_Window(BOXSIZE*8+1+2*BORDER, BOXSIZE*32+1+2*BORDER) {
   clear_border();
   set_modal();
-  initial = which = previous = oldcol;
-  done = 0;
+  initial = which = oldcol;
 }
 
 void ColorMenu::drawbox(Fl_Color c) {
@@ -82,7 +82,7 @@ void ColorMenu::draw() {
 }
 
 int ColorMenu::handle(int e) {
-  int c = which;
+  Fl_Color c = which;
   switch (e) {
   case FL_PUSH:
   case FL_DRAG: {
@@ -148,7 +148,7 @@ Fl_Color ColorMenu::run() {
   Fl::grab(*this);
   done = 0;
   while (!done) Fl::wait();
-  Fl::release();
+  Fl::grab(0);
   return which;
 }
 
@@ -158,5 +158,5 @@ Fl_Color fl_show_colormap(Fl_Color oldcol) {
 }
 
 //
-// End of "$Id: fl_show_colormap.cxx 6926 2009-11-07 21:01:08Z matt $".
+// End of "$Id: fl_show_colormap.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
