@@ -1,9 +1,9 @@
 //
-// "$Id: forms_fselect.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: forms_fselect.cxx 8063 2010-12-19 21:20:10Z matt $"
 //
 // Forms file selection routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,9 +30,9 @@
 #include <FL/forms.H>
 #include "flstring.h"
 
-static char fl_directory[1024];
-static const char *fl_pattern;  // assummed passed value is static
-static char fl_filename[1024];
+static char fl_directory[FL_PATH_MAX];
+static const char *fl_pattern;  // assumed passed value is static
+static char fl_filename[FL_PATH_MAX];
 
 char* fl_show_file_selector(const char *message,const char *dir,
 			    const char *pat,const char *fname) {
@@ -63,5 +63,5 @@ char*	fl_get_pattern() {return (char *)fl_pattern;}
 char*	fl_get_filename() {return fl_filename;}
 
 //
-// End of "$Id: forms_fselect.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: forms_fselect.cxx 8063 2010-12-19 21:20:10Z matt $".
 //
