@@ -1,9 +1,9 @@
 //
-// "$Id: forms_pixmap.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id: forms_pixmap.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Forms pixmap drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -27,15 +27,26 @@
 
 #include <FL/forms.H>
 
+/**
+  Creates a new Fl_FormsPixmap widget using the given box type, position,
+  size and label string.
+  \param[in] t box type
+  \param[in] X, Y, W, H position and size
+  \param[in] L widget label, default is no label
+*/
 Fl_FormsPixmap::Fl_FormsPixmap(
-  Fl_Boxtype t, int X, int Y, int W, int H, const char* l)
-: Fl_Widget(X, Y, W, H, l) {
+  Fl_Boxtype t, int X, int Y, int W, int H, const char* L)
+: Fl_Widget(X, Y, W, H, L) {
   box(t);
   b = 0;
   color(FL_BLACK);
   align(FL_ALIGN_BOTTOM);
 }
 
+/**
+  Set/create the internal pixmap using raw data.
+  \param[in] bits raw data
+*/
 void Fl_FormsPixmap::set(char*const* bits) {
   delete b;
   b = new Fl_Pixmap(bits);
@@ -48,5 +59,5 @@ void Fl_FormsPixmap::draw() {
 }
 
 //
-// End of "$Id: forms_pixmap.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id: forms_pixmap.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
