@@ -1,9 +1,9 @@
 //
-// "$Id: gl_start.cxx,v 1.6.2.5.2.11 2004/09/09 21:34:48 matthiaswm Exp $"
+// "$Id: gl_start.cxx 8368 2011-02-04 23:32:53Z manolo $"
 //
 // OpenGL context routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // You MUST use gl_visual() to select the default visual before doing
@@ -44,8 +46,6 @@
 #include <FL/fl_draw.H>
 #include "Fl_Gl_Choice.H"
 
-extern int fl_clip_state_number; // in fl_rect.cxx
-
 static GLContext context;
 static int clip_state_number=-1;
 static int pw, ph;
@@ -60,19 +60,19 @@ static Fl_Gl_Choice* gl_choice;
 
 Fl_Region XRectangleRegion(int x, int y, int w, int h); // in fl_rect.cxx
 
+/** Creates an OpenGL context */
 void gl_start() {
   if (!context) {
-#ifdef WIN32
+#if defined(USE_X11)
+    context = fl_create_gl_context(fl_visual);
+#elif defined(WIN32)
     if (!gl_choice) Fl::gl_visual(0);
     context = fl_create_gl_context(Fl_Window::current(), gl_choice);
-#elif defined(__APPLE_QD__)
-    // \todo Mac : We need to check the code and verify it with Apple Sample code. The 'shiny'-test should at least work with the software OpenGL emulator
-    context = fl_create_gl_context(Fl_Window::current(), gl_choice);
 #elif defined(__APPLE_QUARTZ__)
     // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
     context = fl_create_gl_context(Fl_Window::current(), gl_choice);
 #else
-    context = fl_create_gl_context(fl_visual);
+#  error Unsupported platform
 #endif
   }
   fl_set_gl_context(Fl_Window::current(), context);
@@ -87,8 +87,8 @@ void gl_start() {
     glOrtho(0, pw, 0, ph, -1, 1);
     glDrawBuffer(GL_FRONT);
   }
-  if (clip_state_number != fl_clip_state_number) {
-    clip_state_number = fl_clip_state_number;
+  if (clip_state_number != fl_graphics_driver->fl_clip_state_number) {
+    clip_state_number = fl_graphics_driver->fl_clip_state_number;
     int x, y, w, h;
     if (fl_clip_box(0, 0, Fl_Window::current()->w(), Fl_Window::current()->h(),
 		    x, y, w, h)) {
@@ -101,6 +101,7 @@ void gl_start() {
   }
 }
 
+/** Releases an OpenGL context */
 void gl_finish() {
   glFlush();
 #if !defined(WIN32) && !defined(__APPLE__)
@@ -111,22 +112,21 @@ void gl_finish() {
 int Fl::gl_visual(int mode, int *alist) {
   Fl_Gl_Choice *c = Fl_Gl_Choice::find(mode,alist);
   if (!c) return 0;
-#ifdef WIN32
-  gl_choice = c;
-#elif defined(__APPLE_QD__)
+#if defined(USE_X11)
+  fl_visual = c->vis;
+  fl_colormap = c->colormap;
+#elif defined(WIN32)
   gl_choice = c;
 #elif defined(__APPLE_QUARTZ__)
   // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
   gl_choice = c;
 #else
-  fl_visual = c->vis;
-  fl_colormap = c->colormap;
+#  error Unsupported platform
 #endif
   return 1;
 }
-
 #endif
 
 //
-// End of "$Id: gl_start.cxx,v 1.6.2.5.2.11 2004/09/09 21:34:48 matthiaswm Exp $".
+// End of "$Id: gl_start.cxx 8368 2011-02-04 23:32:53Z manolo $".
 //
