@@ -1,9 +1,9 @@
 //
-// "$Id: arc.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: arc.cxx 7978 2010-12-08 14:00:35Z AlbrechtS $"
 //
 // Arc drawing test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -35,7 +35,7 @@ const char* name[6] = {"X", "Y", "R", "start", "end", "rotate"};
 
 class Drawing : public Fl_Widget {
   void draw() {
-    fl_clip(x(),y(),w(),h());
+    fl_push_clip(x(),y(),w(),h());
     fl_color(FL_DARK3);
     fl_rectf(x(),y(),w(),h());
     fl_push_matrix();
@@ -66,7 +66,7 @@ Drawing *d;
 
 void slider_cb(Fl_Widget* o, void* v) {
   Fl_Slider* s = (Fl_Slider*)o;
-  args[long(v)] = s->value();
+  args[fl_intptr_t(v)] = s->value();
   d->redraw();
 }
 
@@ -94,6 +94,6 @@ int main(int argc, char** argv) {
 
 
 //
-// End of "$Id: arc.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: arc.cxx 7978 2010-12-08 14:00:35Z AlbrechtS $".
 //
 
