@@ -1,9 +1,9 @@
 //
-// "$Id: bitmap.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: bitmap.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Bitmap label test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,7 +26,7 @@
 //
 
 #include <FL/Fl.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Bitmap.H>
 #include <stdio.h>
@@ -102,7 +102,7 @@ static uchar sorceress_bits[] = {
 
 Fl_Toggle_Button *leftb,*rightb,*topb,*bottomb,*insideb,*overb,*inactb;
 Fl_Button *b;
-Fl_Window *w;
+Fl_Double_Window *w;
 
 void button_cb(Fl_Widget *,void *) {
   int i = 0;
@@ -119,7 +119,7 @@ void button_cb(Fl_Widget *,void *) {
 }
 
 int main(int argc, char **argv) {
-  w = new Fl_Window(400,400);
+  w = new Fl_Double_Window(400,400);
   b = new Fl_Button(140,160,120,120,"Bitmap");
   b->image(new Fl_Bitmap(sorceress_bits,sorceress_width,sorceress_height));
   leftb = new Fl_Toggle_Button(25,50,50,25,"left");
@@ -143,5 +143,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: bitmap.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: bitmap.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
