@@ -1,9 +1,9 @@
 //
-// "$Id: blocks.cxx 5976 2007-11-19 14:14:13Z matt $"
+// "$Id: blocks.cxx 7904 2010-11-28 21:12:59Z matt $"
 //
 // "Block Attack!" scrolling blocks game using the Fast Light Tool Kit (FLTK).
 //
-// Copyright 2006 by Michael Sweet.
+// Copyright 2006-2010 by Michael Sweet.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -148,6 +148,9 @@ class BlockSound {
   // Private, OS-specific data...
 #ifdef __APPLE__
   AudioDeviceID device;
+#  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+  AudioDeviceIOProcID audio_proc_id;
+#  endif
   AudioStreamBasicDescription format;
   short *data;
   int remaining;
@@ -216,11 +219,14 @@ BlockSound::BlockSound() {
   // Check we got linear pcm - what to do if we did not ???
   if (format.mFormatID != kAudioFormatLinearPCM) return;
 
-  // Attach the callback
+  // Attach the callback and start the device
+#  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+  if (AudioDeviceCreateIOProcID(device, audio_cb, (void *)this, &audio_proc_id) != noErr) return;
+  AudioDeviceStart(device, audio_proc_id);
+#  else
   if (AudioDeviceAddIOProc(device, audio_cb, (void *)this) != noErr) return;
-
-  // Start the device...
   AudioDeviceStart(device, audio_cb);
+#  endif
 
   sample_size = (int)format.mSampleRate;
 
@@ -312,8 +318,13 @@ BlockSound::BlockSound() {
 BlockSound::~BlockSound() {
 #ifdef __APPLE__
   if (sample_size) {
+#  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+    AudioDeviceStop(device, audio_proc_id);
+    AudioDeviceDestroyIOProcID(device, audio_proc_id);
+#  else
     AudioDeviceStop(device, audio_cb);
     AudioDeviceRemoveIOProc(device, audio_cb);
+#  endif
   }
 
 #elif defined(WIN32)
@@ -1015,5 +1026,5 @@ BlockWindow::timeout_cb(BlockWindow *bw) {
 
 
 //
-// End of "$Id: blocks.cxx 5976 2007-11-19 14:14:13Z matt $".
+// End of "$Id: blocks.cxx 7904 2010-11-28 21:12:59Z matt $".
 //
