@@ -1,9 +1,9 @@
 //
-// "$Id: demo.cxx 5712 2007-02-23 19:27:43Z matt $"
+// "$Id: demo.cxx 8252 2011-01-11 10:36:44Z manolo $"
 //
 // Main demo program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2007 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -36,11 +36,17 @@
 #    define chdir _chdir
 #    define putenv _putenv
 #  endif // !__WATCOMC__
+#elif defined USING_XCODE
+#include <ApplicationServices/ApplicationServices.h>
+#include <unistd.h> // for chdir()
+#include <stdio.h>
+#include <stdlib.h> // for system()
+#include <string.h>
 #else
 #  include <unistd.h>
 #endif
 #include <FL/Fl.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Choice.H>
@@ -56,22 +62,22 @@ void doscheme(Fl_Choice *c, void *) {
   Fl::scheme(c->text(c->value()));
 }
 
-Fl_Window *form;
+Fl_Double_Window *form;
 Fl_Button *but[9];
 
 void create_the_forms() {
   Fl_Widget *obj;
-  form = new Fl_Window(350, 440);
-  obj = new Fl_Box(FL_FRAME_BOX,10,385,330,40,"FLTK Demonstration");
+  form = new Fl_Double_Window(350, 440);
+  obj = new Fl_Box(FL_FRAME_BOX,10,15,330,40,"FLTK Demonstration");
   obj->color(FL_GRAY-4);
   obj->labelsize(24);
   obj->labelfont(FL_BOLD);
   obj->labeltype(FL_ENGRAVED_LABEL);
-  obj = new Fl_Box(FL_FRAME_BOX,10,45,330,330,0);
+  obj = new Fl_Box(FL_FRAME_BOX,10,65,330,330,0);
   obj->color(FL_GRAY-8);
-  obj = new Fl_Button(280,10,60,25,"Exit");
+  obj = new Fl_Button(280,405,60,25,"Exit");
   obj->callback(doexit);
-  Fl_Choice *choice = new Fl_Choice(75, 10, 100, 25, "Scheme:");
+  Fl_Choice *choice = new Fl_Choice(75, 405, 100, 25, "Scheme:");
   choice->labelfont(FL_HELVETICA_BOLD);
   choice->add("none");
   choice->add("gtk+");
@@ -81,22 +87,22 @@ void create_the_forms() {
   if (!Fl::scheme()) choice->value(0);
   else if (!strcmp(Fl::scheme(), "gtk+")) choice->value(1);
   else choice->value(2);
-  obj = new Fl_Button(10,45,330,380); obj->type(FL_HIDDEN_BUTTON);
+  obj = new Fl_Button(10,15,330,380); obj->type(FL_HIDDEN_BUTTON);
   obj->callback(doback);
-  obj = but[0] = new Fl_Button(30,265,90,90);
-  obj = but[1] = new Fl_Button(130,265,90,90);
-  obj = but[2] = new Fl_Button(230,265,90,90);
-  obj = but[5] = new Fl_Button(230,165,90,90);
-  obj = but[4] = new Fl_Button(130,165,90,90);
-  obj = but[3] = new Fl_Button(30,165,90,90);
-  obj = but[6] = new Fl_Button(30,65,90,90);
-  obj = but[7] = new Fl_Button(130,65,90,90);
-  obj = but[8] = new Fl_Button(230,65,90,90);
+  obj = but[0] = new Fl_Button( 30, 85,90,90);
+  obj = but[1] = new Fl_Button(130, 85,90,90);
+  obj = but[2] = new Fl_Button(230, 85,90,90);
+  obj = but[3] = new Fl_Button( 30,185,90,90);
+  obj = but[4] = new Fl_Button(130,185,90,90);
+  obj = but[5] = new Fl_Button(230,185,90,90);
+  obj = but[6] = new Fl_Button( 30,285,90,90);
+  obj = but[7] = new Fl_Button(130,285,90,90);
+  obj = but[8] = new Fl_Button(230,285,90,90);
   for (int i=0; i<9; i++) {
     but[i]->align(FL_ALIGN_WRAP);
     but[i]->callback(dobut, i);
   }
-  form->forms_end();
+  form->end();
 }
 
 /* Maintaining and building up the menus. */
@@ -154,7 +160,7 @@ int b2n[][9] = {
 	{  0, -1,  1,  2,  3,  4,  5, -1,  6},
 	{  0,  1,  2,  3, -1,  4,  5,  6,  7},
 	{  0,  1,  2,  3,  4,  5,  6,  7,  8}
-  };
+};
 int n2b[][9] = { 
 	{  4, -1, -1, -1, -1, -1, -1, -1, -1},
 	{  3,  5, -1, -1, -1, -1, -1, -1, -1},
@@ -165,17 +171,17 @@ int n2b[][9] = {
 	{  0,  2,  3,  4,  5,  6,  8, -1, -1},
 	{  0,  1,  2,  3,  5,  6,  7,  8, -1},
 	{  0,  1,  2,  3,  4,  5,  6,  7,  8}
-  };
+};
 
 int but2numb(int bnumb, int maxnumb)
 /* Transforms a button number to an item number when there are
-   maxnumb items in total. -1 if the button should not exist. */
- { return b2n[maxnumb][bnumb]; }
+ maxnumb items in total. -1 if the button should not exist. */
+{ return b2n[maxnumb][bnumb]; }
 
 int numb2but(int inumb, int maxnumb)
 /* Transforms an item number to a button number when there are
-   maxnumb items in total. -1 if the item should not exist. */
- { return n2b[maxnumb][inumb]; }
+ maxnumb items in total. -1 if the item should not exist. */
+{ return n2b[maxnumb][inumb]; }
 
 /* Pushing and Popping menus */
 
@@ -222,33 +228,33 @@ void dobut(Fl_Widget *, long arg)
   if (menus[men].icommand[bn][0] == '@')
     push_menu(menus[men].icommand[bn]);
   else {
-
+    
 #ifdef WIN32
     STARTUPINFO		suInfo;		// Process startup information
     PROCESS_INFORMATION	prInfo;		// Process information
-
+    
     memset(&suInfo, 0, sizeof(suInfo));
     suInfo.cb = sizeof(suInfo);
-
+    
     int icommand_length = strlen(menus[men].icommand[bn]);
-
+    
     char* copy_of_icommand = new char[icommand_length+1];
     strcpy(copy_of_icommand,menus[men].icommand[bn]);
-
+    
     // On WIN32 the .exe suffix needs to be appended to the command
     // whilst leaving any additional parameters unchanged - this
     // is required to handle the correct conversion of cases such as : 
     // `../fluid/fluid valuators.fl' to '../fluid/fluid.exe valuators.fl'.
-
+    
     // skip leading spaces.
     char* start_command = copy_of_icommand;
     while(*start_command == ' ') ++start_command;
-
+    
     // find the space between the command and parameters if one exists.
     char* start_parameters = strchr(start_command,' ');
-
+    
     char* command = new char[icommand_length+6]; // 6 for extra 'd.exe\0'
-
+    
     if (start_parameters==NULL) { // no parameters required.
 #  ifdef _DEBUG
       sprintf(command, "%sd.exe", start_command);
@@ -261,28 +267,64 @@ void dobut(Fl_Widget *, long arg)
       *start_parameters = 0;
       // move start_paremeters to skip over the intermediate space.
       ++start_parameters;
-
+      
 #  ifdef _DEBUG
       sprintf(command, "%sd.exe %s", start_command, start_parameters);
 #  else
       sprintf(command, "%s.exe %s", start_command, start_parameters);
 #  endif // _DEBUG
     }
-
+    
     CreateProcess(NULL, command, NULL, NULL, FALSE,
                   NORMAL_PRIORITY_CLASS, NULL, NULL, &suInfo, &prInfo);
-	
+    
     delete[] command;
     delete[] copy_of_icommand;
-	
+    
+#elif defined USING_XCODE
+    char *cmd = strdup(menus[men].icommand[bn]);
+    char *arg = strchr(cmd, ' ');
+    
+    char command[2048], path[2048], app_path[2048];
+    
+    // this neat litle block of code ensures that the current directory is set 
+    // to the location of the Demo application.
+    CFBundleRef app = CFBundleGetMainBundle();
+    CFURLRef url = CFBundleCopyBundleURL(app);    
+    CFStringRef cc_app_path = CFURLCopyFileSystemPath(url, kCFURLPOSIXPathStyle);
+    CFStringGetCString(cc_app_path, app_path, 2048, kCFStringEncodingUTF8);
+    if (*app_path) {
+      char *n = strrchr(app_path, '/');
+      if (n) {
+        *n = 0;
+        chdir(app_path);
+      }
+    }
+    
+    if (arg) {
+      *arg = 0;
+      if (strcmp(cmd, "../fluid/fluid")==0) {
+        fl_filename_absolute(path, 2048, "../../../../test/");
+	sprintf(command, "open Fluid.app --args %s%s", path, arg+1);
+      } else {
+        fl_filename_absolute(path, 2048, "../../../../test/");
+	sprintf(command, "open %s.app --args %s%s", cmd, path, arg+1);
+      }
+    } else {
+      sprintf(command, "open %s.app", cmd);
+    }
+//    puts(command);    
+    system(command);
+    
+    free(cmd);
 #else // NON WIN32 systems.
-
+    
     int icommand_length = strlen(menus[men].icommand[bn]);
     char* command = new char[icommand_length+5]; // 5 for extra './' and ' &\0' 
-
+    
     sprintf(command, "./%s &", menus[men].icommand[bn]);
-    system(command);
-
+    if (system(command)==-1) { /* ignore */ }
+    
     delete[] command;
 #endif // WIN32
   }
@@ -295,13 +337,23 @@ void doexit(Fl_Widget *, void *) {exit(0);}
 int load_the_menu(const char* fname)
 /* Loads the menu file. Returns whether successful. */
 {
-  FILE *fin;
+  FILE *fin = 0;
   char line[256], mname[64],iname[64],cname[64];
-  int i,j;
-  fin = fopen(fname,"r");
-  if (fin == NULL)
-  {
-//    fl_show_message("ERROR","","Cannot read the menu description file.");
+  int i, j;
+  fin = fl_fopen(fname,"r");
+#if defined ( USING_XCODE )
+  if (fin == NULL) {
+    // mac os bundle menu detection:
+    char* pos = strrchr(fname,'/');
+    if (!pos) return 0;
+    *pos='\0';
+    pos = strrchr(fname,'/');
+    if (!pos) return 0;
+    strcpy(pos,"/Resources/demo.menu");
+    fin  = fl_fopen(fname,"r");
+  }
+#endif
+  if (fin == NULL) {
     return 0;
   }
   for (;;) {
@@ -325,10 +377,10 @@ int load_the_menu(const char* fname)
     while (line[i] != ':' && line[i] != '\n')
     {
       if (line[i] == '\\') {
-	i++;
-	if (line[i] == 'n') iname[j++] = '\n';
-	else iname[j++] = line[i];
-	i++;
+        i++;
+        if (line[i] == 'n') iname[j++] = '\n';
+        else iname[j++] = line[i];
+        i++;
       } else
         iname[j++] = line[i++];
     }
@@ -344,8 +396,8 @@ int load_the_menu(const char* fname)
 }
 
 int main(int argc, char **argv) {
-  putenv((char *)"FLTK_DOCDIR=../documentation");
-  char buf[256];
+  putenv((char *)"FLTK_DOCDIR=../documentation/html");
+  char buf[FL_PATH_MAX];
   strcpy(buf, argv[0]);
 #if ( defined _MSC_VER || defined __MWERKS__ ) && defined _DEBUG
   // MS_VisualC appends a 'd' to debugging executables. remove it.
@@ -358,14 +410,17 @@ int main(int argc, char **argv) {
   if (!Fl::args(argc,argv,i) || i < argc-1)
     Fl::fatal("Usage: %s <switches> <menufile>\n%s",argv[0],Fl::help);
   if (i < argc) fname = argv[i];
-
+  
   create_the_forms();
-
+  
   if (!load_the_menu(fname)) Fl::fatal("Can't open %s",fname);
   if (buf!=fname)
     strcpy(buf,fname);
   const char *c = fl_filename_name(buf);
-  if (c > buf) {buf[c-buf] = 0; chdir(buf);}
+  if (c > buf) {
+    buf[c-buf] = 0; 
+    if (chdir(buf)==-1) { /* ignore */ }
+  }
   push_menu("@main");
   form->show(argc,argv);
   Fl::run();
@@ -373,6 +428,6 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: demo.cxx 5712 2007-02-23 19:27:43Z matt $".
+// End of "$Id: demo.cxx 8252 2011-01-11 10:36:44Z manolo $".
 //
 
