@@ -1,5 +1,5 @@
 //
-// "$Id: forms.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: forms.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Another forms demo for the Fast Light Tool Kit (FLTK).
 //
@@ -9,7 +9,7 @@
 // This demo show the different boxtypes. Note that some
 // boxtypes are not appropriate for some objects
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -65,6 +65,8 @@ static VN_struct btypes[]=
    {FL_OVAL3D_DOWNBOX,"oval3d downbox"},
    {FL_PLASTIC_UP_BOX,"plastic upbox"},
    {FL_PLASTIC_DOWN_BOX,"plastic downbox"},
+   {FL_GTK_UP_BOX,"GTK up box"},
+   {FL_GTK_ROUND_UP_BOX,"GTK round up box"},
    /* sentinel */
    {-1}
 };
@@ -233,5 +235,5 @@ main (int argc, char *argv[])
 }
 
 //
-// End of "$Id: forms.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: forms.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
