@@ -1,9 +1,9 @@
 //
-// "$Id: gl_overlay.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: gl_overlay.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $"
 //
 // OpenGL overlay test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,7 +25,7 @@
 //     http://www.fltk.org/str.php
 //
 
-#include "config.h"
+#include <config.h>
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Hor_Slider.H>
@@ -72,9 +72,9 @@ void shape_window::draw() {
   glClear(GL_COLOR_BUFFER_BIT);
   //  for (int j=1; j<=1000; j++) {
     glBegin(GL_POLYGON);
-    for (int i=0; i<sides; i++) {
-      double ang = i*2*M_PI/sides;
-      glColor3f(float(i)/sides,float(i)/sides,float(i)/sides);
+    for (int j=0; j<sides; j++) {
+      double ang = j*2*M_PI/sides;
+      glColor3f(float(j)/sides,float(j)/sides,float(j)/sides);
       glVertex3f(cos(ang),sin(ang),0);
     }
     glEnd();
@@ -92,8 +92,8 @@ void shape_window::draw_overlay() {
 // draw an amazing graphic:
   gl_color(FL_RED);
   glBegin(GL_LINE_LOOP);
-  for (int i=0; i<overlay_sides; i++) {
-    double ang = i*2*M_PI/overlay_sides;
+  for (int j=0; j<overlay_sides; j++) {
+    double ang = j*2*M_PI/overlay_sides;
     glVertex3f(cos(ang),sin(ang),0);
   }
   glEnd();
@@ -153,5 +153,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: gl_overlay.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: gl_overlay.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $".
 //
