@@ -1,5 +1,5 @@
 //
-// "$Id: image.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: image.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Fl_Image test program for the Fast Light Tool Kit (FLTK).
 //
@@ -7,7 +7,7 @@
 // as an icon or postage stamp.  Use fl_draw_image to go directly
 // from an buffered image that changes often.
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,7 +30,7 @@
 //
 
 #include <FL/Fl.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Image.H>
 #include <stdio.h>
@@ -65,7 +65,7 @@ void make_image() {
 
 Fl_Toggle_Button *leftb,*rightb,*topb,*bottomb,*insideb,*overb,*inactb;
 Fl_Button *b;
-Fl_Window *w;
+Fl_Double_Window *w;
 
 void button_cb(Fl_Widget *,void *) {
   int i = 0;
@@ -121,7 +121,7 @@ int main(int argc, char **argv) {
   }
 #endif
 
-  Fl_Window window(400,400); ::w = &window;
+  Fl_Double_Window window(400,400); ::w = &window;
   window.color(FL_WHITE);
   Fl_Button b(140,160,120,120,"Image w/Alpha"); ::b = &b;
 
@@ -157,5 +157,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: image.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: image.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
