@@ -1,5 +1,5 @@
 //
-// "$Id: keyboard.cxx 5702 2007-02-21 15:49:38Z matt $"
+// "$Id: keyboard.cxx 7978 2010-12-08 14:00:35Z AlbrechtS $"
 //
 // Keyboard/event test program for the Fast Light Tool Kit (FLTK).
 //
@@ -17,7 +17,7 @@
 //
 // On IRIX the backslash key does not work.  A bug in XKeysymToKeycode?
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -100,7 +100,7 @@ struct keycode_table{int n; const char* text;} table[] = {
 
 int main(int argc, char** argv) {
   Fl::add_handler(handle);
-  Fl_Window *window = make_window();
+  MyWindow *window = make_window();
   window->show(argc,argv);
   while (Fl::wait()) {
     const char *str;
@@ -109,14 +109,14 @@ int main(int argc, char** argv) {
     for (int i = 0; i < window->children(); i++) {
       Fl_Widget* b = window->child(i);
       if (b->callback() == (Fl_Callback*)key_cb) {
-	int i = (long)b->user_data();
+	int i = b->argument();
 	if (!i) i = b->label()[0];
         Fl_Button *btn = ((Fl_Button*)b);
         int state = Fl::event_key(i);
         if (btn->value()!=state)
 	  btn->value(state);
       } else if (b->callback() == (Fl_Callback*)shift_cb) {
-	int i = (long)b->user_data();
+	int i = b->argument();
         Fl_Button *btn = ((Fl_Button*)b);
         int state = Fl::event_state(i);
         if (btn->value()!=state)
@@ -154,5 +154,5 @@ int main(int argc, char** argv) {
 }
 
 //
-// End of "$Id: keyboard.cxx 5702 2007-02-21 15:49:38Z matt $".
+// End of "$Id: keyboard.cxx 7978 2010-12-08 14:00:35Z AlbrechtS $".
 //
