@@ -1,9 +1,9 @@
 //
-// "$Id: line_style.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: line_style.cxx 7982 2010-12-08 23:58:32Z AlbrechtS $"
 //
 // Line style demo for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 2000-2005 by Bill Spitzak and others.
+// Copyright 2000-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,48 +26,61 @@
 //
 
 #include <FL/Fl.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Value_Slider.H>
 #include <FL/fl_draw.H>
 #include <FL/Fl_Choice.H>
+#include <FL/Fl_Check_Button.H>
+#include <FL/Fl_Box.H>
 
-Fl_Window *form;
-Fl_Slider *sliders[8];
+Fl_Double_Window *form;
+Fl_Slider *sliders[9];
 Fl_Choice *choice[3];
+Fl_Check_Button *draw_line;
 
-class test_box: public Fl_Window {
+class test_box: public Fl_Double_Window {
   void draw();
 public:
   test_box(int x,int y,int w,int h,const char *l=0)
-    : Fl_Window(x,y,w,h,l) {}
+    : Fl_Double_Window(x,y,w,h,l) {}
 }*test;
 
 void test_box::draw() {
-  Fl_Window::draw();
+  Fl_Double_Window::draw();
   fl_color((uchar)(sliders[0]->value()),
 	   (uchar)(sliders[1]->value()),
 	   (uchar)(sliders[2]->value()));
-  char buf[5];
-  buf[0] = char(sliders[4]->value());
-  buf[1] = char(sliders[5]->value());
-  buf[2] = char(sliders[6]->value());
-  buf[3] = char(sliders[7]->value());
-  buf[4] = 0;
+  // dashes
+  char dashes[5];
+  dashes[0] = char(sliders[5]->value());
+  dashes[1] = char(sliders[6]->value());
+  dashes[2] = char(sliders[7]->value());
+  dashes[3] = char(sliders[8]->value());
+  dashes[4] = 0;
   fl_line_style(
-    (long)(choice[0]->mvalue()->user_data()) +
-    (long)(choice[1]->mvalue()->user_data()) +
-    (long)(choice[2]->mvalue()->user_data()),
-    (long)(sliders[3]->value()),
-    buf);
-  fl_rect(10,10,w()-20,h()-20);
-  fl_begin_line();
-  fl_vertex(35, 35);
-  fl_vertex(w()-35, h()-35);
-  fl_vertex(w()-40, 35);
-  fl_vertex(35, h()/2);
-  fl_end_line();
-  // you must reset the line type when done:
-  fl_line_style(FL_SOLID);
+    choice[0]->mvalue()->argument() +
+    choice[1]->mvalue()->argument() +
+    choice[2]->mvalue()->argument(),
+    long(sliders[3]->value()),		// width
+    dashes);
+
+  // draw the defined fl_rect and fl_vertex first and then
+  // the additional one-pixel line, if enabled
+  // sliders[4] = x/y coordinate translation (default = 10)
+
+  for (int i=0; i<draw_line->value()+1; i++) {
+    int move = (int)sliders[4]->value();
+    fl_rect(move,move,w()-20,h()-20);
+    fl_begin_line();
+    fl_vertex(move+25, move+25);
+    fl_vertex(w()-45+move, h()-45+move);
+    fl_vertex(w()-50+move, move+25);
+    fl_vertex(move+25, h()/2-10+move);
+    fl_end_line();
+    // you must reset the line type when done:
+    fl_line_style(FL_SOLID);
+    fl_color(FL_BLACK);
+  }
 }
 
 Fl_Menu_Item style_menu[] = {
@@ -101,7 +114,7 @@ void do_redraw(Fl_Widget*,void*)
 }
 
 void makeform(const char *) {
-  form = new Fl_Window(500,210,"fl_line_style() test");
+  form = new Fl_Double_Window(500,250,"fl_line_style() test");
   sliders[0]= new Fl_Value_Slider(280,10,180,20,"R");
   sliders[0]->bounds(0,255);
   sliders[1]= new Fl_Value_Slider(280,30,180,20,"G");
@@ -116,27 +129,38 @@ void makeform(const char *) {
   choice[2]->menu(join_menu);
   sliders[3]= new Fl_Value_Slider(280,130,180,20,"Width");
   sliders[3]->bounds(0,20);
-  sliders[4] = new Fl_Slider(200,170,70,20,"Dash");
-  sliders[4]->align(FL_ALIGN_TOP_LEFT);
-  sliders[4]->bounds(0,40);
-  sliders[5] = new Fl_Slider(270,170,70,20);
+  sliders[4]= new Fl_Value_Slider(280,150,180,20,"Move");
+  sliders[4]->bounds(-10,20);
+  draw_line = new Fl_Check_Button(280,170,20,20,"&Line");
+  draw_line->align(FL_ALIGN_LEFT);
+  new Fl_Box (305,170,160,20,"add a 1-pixel black line");
+  sliders[5] = new Fl_Slider(200,210,70,20,"Dash");
+  sliders[5]->align(FL_ALIGN_TOP_LEFT);
   sliders[5]->bounds(0,40);
-  sliders[6] = new Fl_Slider(340,170,70,20);
+  sliders[6] = new Fl_Slider(270,210,70,20);
   sliders[6]->bounds(0,40);
-  sliders[7] = new Fl_Slider(410,170,70,20);
+  sliders[7] = new Fl_Slider(340,210,70,20);
   sliders[7]->bounds(0,40);
+  sliders[8] = new Fl_Slider(410,210,70,20);
+  sliders[8]->bounds(0,40);
   int i;
-  for (i=0;i<8;i++) {
+  for (i=0;i<9;i++) {
     sliders[i]->type(1);
-    if (i<4) sliders[i]->align(FL_ALIGN_LEFT);
+    if (i<5) sliders[i]->align(FL_ALIGN_LEFT);
     sliders[i]->callback((Fl_Callback*)do_redraw);
     sliders[i]->step(1);
   }
+  sliders[0]->value(255); // R
+  sliders[1]->value(100); // G
+  sliders[2]->value(100); // B
+  sliders[4]->value(10);  // move line coordinates
+  draw_line->value(0);
+  draw_line->callback((Fl_Callback*)do_redraw);
   for (i=0;i<3;i++) {
     choice[i]->value(0);
     choice[i]->callback((Fl_Callback*)do_redraw);
   }
-  test=new test_box(10,10,190,190);
+  test = new test_box(0,0,200,200);
   test->end();
   form->resizable(test);
   form->end();
@@ -148,7 +172,6 @@ int main(int argc, char **argv) {
   return Fl::run();
 }
 
-
 //
-// End of "$Id: line_style.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: line_style.cxx 7982 2010-12-08 23:58:32Z AlbrechtS $".
 //
