@@ -1,5 +1,5 @@
 //
-// "$Id: list_visuals.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: list_visuals.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $"
 //
 // Visual list utility for the Fast Light Tool Kit (FLTK).
 //
@@ -11,7 +11,7 @@
 // This file may be #included in another program to make a function to
 // call to list the visuals.  Fl.H must be included first to indicate this.
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -44,7 +44,7 @@ int main(int, char**) {
 
 #else
 
-#include "config.h"
+#include <config.h>
 
 #ifndef Fl_H
 
@@ -237,5 +237,5 @@ int main(int argc, char **argv) {
 #endif
 
 //
-// End of "$Id: list_visuals.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: list_visuals.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $".
 //
