@@ -1,9 +1,9 @@
 //
-// "$Id: mandelbrot.cxx 6474 2008-10-19 20:15:48Z matt $"
+// "$Id: mandelbrot.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Mandelbrot set demo for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -27,6 +27,8 @@
 
 #include "mandelbrot_ui.h"
 #include <FL/fl_draw.H>
+#include <FL/Fl_Button.H>
+#include <FL/Fl_Printer.H>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -43,8 +45,30 @@ void set_idle() {
 
 static void window_callback(Fl_Widget*, void*) {exit(0);}
 
+static void print(Fl_Widget *o, void *data)
+{
+  Fl_Printer printer;
+  Fl_Window *win = o->window();
+  if(!win->visible()) return;
+  win->make_current();
+  uchar *image_data = fl_read_image(NULL, 0, 0, win->w(), win->h(), 0);
+  if( printer.start_job(1) ) return;
+  if( printer.start_page() ) return;
+  printer.scale(.7,.7);
+  fl_draw_image(image_data, 0,0, win->w(), win->h());
+  printer.end_page();
+  delete image_data;
+  printer.end_job();
+}
+
 int main(int argc, char **argv) {
   mbrot.make_window();
+    mbrot.window->begin();
+    Fl_Button* o = new Fl_Button(0, 0, 0, 0, NULL);
+    o->callback(print,NULL);
+    o->shortcut(FL_CTRL+'p');
+    mbrot.window->end();
+
   mbrot.d->X = -.75;
   mbrot.d->scale = 2.5;
   mbrot.update_label();
@@ -199,5 +223,5 @@ void Drawing_Area::resize(int XX,int YY,int WW,int HH) {
 }
 
 //
-// End of "$Id: mandelbrot.cxx 6474 2008-10-19 20:15:48Z matt $".
+// End of "$Id: mandelbrot.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
