@@ -1,12 +1,12 @@
 //
-// "$Id: navigation.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: navigation.cxx 8090 2010-12-21 02:37:41Z greg.ercolano $"
 //
 // Navigation test program for the Fast Light Tool Kit (FLTK).
 //
 // Silly test of navigation keys. This is not a recommended method of
 // laying out your panels!
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -33,14 +33,25 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Input.H>
+#include <FL/Fl_Light_Button.H>
 
 #define WIDTH 600
 #define HEIGHT 300
 #define GRID 25
 
+void ToggleArrowFocus_CB(Fl_Widget *w, void*) {
+  Fl_Light_Button *b = (Fl_Light_Button*)w;
+  Fl::option(Fl::OPTION_ARROW_FOCUS, b->value() ? true : false);
+}
 int main(int argc, char **argv) {
   if (argc > 1) srand(atoi(argv[1]));
-  Fl_Window window(WIDTH,HEIGHT,argv[0]);
+  Fl_Window window(WIDTH,HEIGHT+40,argv[0]);
+    // Include a toggle button to control arrow focus
+    Fl_Light_Button arrowfocus_butt(10,HEIGHT+10,130,20," Arrow Focus");
+    arrowfocus_butt.callback(ToggleArrowFocus_CB);
+    arrowfocus_butt.value(Fl::option(Fl::OPTION_ARROW_FOCUS) ? 1 : 0);	// use default
+    arrowfocus_butt.tooltip("Control horizontal arrow key focus navigation behavior.\n"
+                            "e.g. Fl::OPTION_ARROW_FOCUS");
   window.end(); // don't auto-add children
   for (int i = 0; i<10000; i++) {
     // make up a random size of widget:
@@ -57,7 +68,7 @@ int main(int argc, char **argv) {
       Fl_Widget *o = window.child(n);
       if (x<o->x()+o->w() && x+w>o->x() &&
 	  y<o->y()+o->h() && y+h>o->y()) break;
-      if (!j && (y < o->y() || y == o->y() && x < o->x())) j = o;
+      if ( !j && ( y<o->y() || (y==o->y() && x<o->x()) ) ) j = o;
     }
     // skip if intersection:
     if (n < window.children()) continue;
@@ -68,5 +79,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: navigation.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: navigation.cxx 8090 2010-12-21 02:37:41Z greg.ercolano $".
 //
