@@ -1,9 +1,9 @@
 //
-// "$Id: shape.cxx 5845 2007-05-20 00:01:06Z mike $"
+// "$Id: shape.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $"
 //
 // Tiny OpenGL demo program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,7 +25,7 @@
 //     http://www.fltk.org/str.php
 //
 
-#include "config.h"
+#include <config.h>
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Hor_Slider.H>
@@ -113,5 +113,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: shape.cxx 5845 2007-05-20 00:01:06Z mike $".
+// End of "$Id: shape.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $".
 //
