@@ -1,9 +1,9 @@
 //
-// "$Id: sudoku.cxx 6973 2009-12-21 16:35:06Z AlbrechtS $"
+// "$Id: sudoku.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Sudoku game using the Fast Light Tool Kit (FLTK).
 //
-// Copyright 2005-2007 by Michael Sweet.
+// Copyright 2005-2010 by Michael Sweet.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 
 #include <FL/Fl.H>
 #include <FL/Enumerations.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Group.H>
 #include <FL/fl_ask.H>
@@ -99,6 +99,9 @@ class SudokuSound {
   // Private, OS-specific data...
 #ifdef __APPLE__
   AudioDeviceID device;
+#  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+  AudioDeviceIOProcID audio_proc_id;
+#  endif
   AudioStreamBasicDescription format;
   short *data;
   int remaining;
@@ -162,7 +165,7 @@ class SudokuCell : public Fl_Widget {
 
 
 // Sudoku window class...
-class Sudoku : public Fl_Window {
+class Sudoku : public Fl_Double_Window {
   Fl_Sys_Menu_Bar *menubar_;
   Fl_Group	*grid_;
   time_t	seed_;
@@ -245,12 +248,15 @@ SudokuSound::SudokuSound() {
   // Check we got linear pcm - what to do if we did not ???
   if (format.mFormatID != kAudioFormatLinearPCM) return;
 
-  // Attach the callback
+  // Attach the callback and start the device
+#  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+  if (AudioDeviceCreateIOProcID(device, audio_cb, (void *)this, &audio_proc_id) != noErr) return;
+  AudioDeviceStart(device, audio_proc_id);
+#  else
   if (AudioDeviceAddIOProc(device, audio_cb, (void *)this) != noErr) return;
-
-  // Start the device...
   AudioDeviceStart(device, audio_cb);
-
+#  endif
+  
   sample_size = (int)format.mSampleRate / 20;
 
 #elif defined(WIN32)
@@ -349,8 +355,13 @@ SudokuSound::SudokuSound() {
 SudokuSound::~SudokuSound() {
 #ifdef __APPLE__
   if (sample_size) {
+#  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+    AudioDeviceStop(device, audio_proc_id);
+    AudioDeviceDestroyIOProcID(device, audio_proc_id);
+#  else
     AudioDeviceStop(device, audio_cb);
     AudioDeviceRemoveIOProc(device, audio_cb);
+#  endif
   }
 
 #elif defined(WIN32)
@@ -627,7 +638,7 @@ Fl_Preferences	Sudoku::prefs_(Fl_Preferences::USER, "fltk.org", "sudoku");
 
 // Create a Sudoku game window...
 Sudoku::Sudoku()
-  : Fl_Window(GROUP_SIZE * 3, GROUP_SIZE * 3 + MENU_OFFSET, "Sudoku")
+  : Fl_Double_Window(GROUP_SIZE * 3, GROUP_SIZE * 3 + MENU_OFFSET, "Sudoku")
 {
   int j, k;
   Fl_Group *g;
@@ -951,7 +962,7 @@ Sudoku::help_cb(Fl_Widget *, void *) {
 	"only once in each column and row. In addition, each 3x3 subgrid\n"
 	"may only contain one of each number.</P>\n"
 
-	"<P>This version of the puzzle is copyright 2005-2006 by Michael R\n"
+	"<P>This version of the puzzle is copyright 2005-2010 by Michael R\n"
 	"Sweet.</P>\n"
 
 	"<P><B>Note:</B> The 'Hard' and 'Impossible' difficulty\n"
@@ -1215,7 +1226,7 @@ Sudoku::reset_cb(Fl_Widget *widget, void *) {
 void
 Sudoku::resize(int X, int Y, int W, int H) {
   // Resize the window...
-  Fl_Window::resize(X, Y, W, H);
+  Fl_Double_Window::resize(X, Y, W, H);
 
   // Save the new window geometry...
   prefs_.set("x", X);
@@ -1331,5 +1342,5 @@ main(int argc, char *argv[]) {
 
 
 //
-// End of "$Id: sudoku.cxx 6973 2009-12-21 16:35:06Z AlbrechtS $".
+// End of "$Id: sudoku.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
