@@ -1,9 +1,9 @@
 //
-// "$Id: symbols.cxx 4392 2005-06-01 20:10:53Z matt $"
+// "$Id: symbols.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Symbol test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,7 +30,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <FL/Fl.H>
-#include <FL/Fl_Single_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Value_Slider.H>
 #include <FL/fl_draw.H>
@@ -41,11 +41,11 @@ int N = 0;
 #define ROWS 6
 #define COLS 6
 
-Fl_Window *window;
+Fl_Double_Window *window;
 Fl_Value_Slider *orientation;
 Fl_Value_Slider *size;
 
-void slider_cb(Fl_Widget *w, void *) {
+void slider_cb(Fl_Widget *, void *) {
   static char buf[80];
   int val = (int)orientation->value();
   int sze = (int)size->value();
@@ -93,7 +93,7 @@ void bt(const char *name) {
 }
 
 int main(int argc, char ** argv) {
-  window = new Fl_Single_Window(COLS*W,ROWS*H+60);
+  window = new Fl_Double_Window(COLS*W,ROWS*H+60);
 bt("@->");
 bt("@>");
 bt("@>>");
@@ -115,7 +115,7 @@ bt("@arrow");
 bt("@returnarrow");
 bt("@square");
 bt("@circle");
-bt("@00010line");
+bt("@line");
 bt("@menu");
 bt("@UpArrow");
 bt("@DnArrow");
@@ -155,5 +155,5 @@ bt("@redo");
 }
 
 //
-// End of "$Id: symbols.cxx 4392 2005-06-01 20:10:53Z matt $".
+// End of "$Id: symbols.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
