@@ -1,9 +1,9 @@
 //
-// "$Id: threads.cxx 5849 2007-05-20 16:32:37Z matt $"
+// "$Id: threads.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $"
 //
 // Threading example program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,11 +25,11 @@
 //     http://www.fltk.org/str.php
 //
 
-#include "config.h"
+#include <config.h>
 
 #if HAVE_PTHREAD || defined(WIN32)
 #  include <FL/Fl.H>
-#  include <FL/Fl_Window.H>
+#  include <FL/Fl_Double_Window.H>
 #  include <FL/Fl_Browser.H>
 #  include <FL/Fl_Value_Output.H>
 #  include <FL/fl_ask.H>
@@ -120,13 +120,13 @@ void* prime_func(void* p)
 
 int main(int argc, char **argv)
 {
-  Fl_Window* w = new Fl_Window(200, 200, "Single Thread");
+  Fl_Double_Window* w = new Fl_Double_Window(200, 200, "Single Thread");
   browser1 = new Fl_Browser(0, 0, 200, 175);
   w->resizable(browser1);
   value1 = new Fl_Value_Output(100, 175, 200, 25, "Max Prime:");
   w->end();
   w->show(argc, argv);
-  w = new Fl_Window(200, 200, "Six Threads");
+  w = new Fl_Double_Window(200, 200, "Six Threads");
   browser2 = new Fl_Browser(0, 0, 200, 175);
   w->resizable(browser2);
   value2 = new Fl_Value_Output(100, 175, 200, 25, "Max Prime:");
@@ -169,5 +169,5 @@ int main() {
 
 
 //
-// End of "$Id: threads.cxx 5849 2007-05-20 16:32:37Z matt $".
+// End of "$Id: threads.cxx 8033 2010-12-15 12:11:16Z AlbrechtS $".
 //
